// EBaseHashT.h: interface for the EBaseHashT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASEHASHT_H__9BDBE5A1_E110_11D2_9222_004005446EE6__INCLUDED_)
#define AFX_EBASEHASHT_H__9BDBE5A1_E110_11D2_9222_004005446EE6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/* Need to work on the iterator...
 */

template<class Key, class Item, class KeyArg, class ItemArg>
class EBaseHashIteratorT;

template<class Key, class Item, class KeyArg, class ItemArg>
class EBaseHashT
{
#ifdef _SUPPORT_TEMPLATE_FRIEND
  friend class EBaseHashIteratorT<Key,Item,KeyArg,ItemArg>;
#endif

public:
  typedef EBaseHashIteratorT<Key,Item,KeyArg,ItemArg> EIterator;

  /*! Constructor.
   * @param nSize initial size of the table.
   */
  EBaseHashT( SL nSize = 3 );

  /*! Destructor
   */
  virtual ~EBaseHashT();

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return reference on item.
   */
  Item& operator[]( KeyArg key );

  /* Lookup a item using its key.
   * @param key key associated to the item.
   * @return reference on item.
   */
//  const Item& operator[]( const KeyArg key ) const;

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return pointeur on item if found. NULL if no item associated to key.
   */
  Item *lookUp( KeyArg key );

  /*! Lookup a item using its key.
   * @param key key associated to the item.
   * @return pointeur on item if found. NULL if no item associated to key.
   */
  const Item *lookUp( KeyArg key ) const;

  /*! Lookup a item key using its key.
   * @param key key associated to the item.
   * @return pointeur on key if found. NULL if no item associated to key.
   */
  const Key *lookUpKey( KeyArg key ) const;

  /*! Add an item to the association table.
   * @param key associated to the item.
   * @param item item to add.
   */
  void add( KeyArg key, ItemArg item );

  /*! Remove an item from the association table.
   * @param key key associated to the item to remove.
   */
  void remove( KeyArg key );

  /*! Remove all the item from the association table.
   */
  void removeAll();

  /*! Resize the table.
   * @param nSize new table size.
   */
  void resize( SL nSize );

  /*! Get the size of the table.
   * @return size of the table.
   */
  SL size() const;

  /*! Get the number of association in the table.
   * @return number of association in the table.
   */
  SL count() const;

  /*! Test if the table is empty.
   * @return true if there is no association in the table, false otherwise.
   */
  bool isEmpty() const;

#ifdef _SUPPORT_TEMPLATE_FRIEND
private:
#endif
  struct ELCAsso
  {
    ELCAsso *_pNext;
    Key _key;
    Item _item;
  };

  EArray<ELCAsso *> _aAsso;

  SL _nNbAsso;

protected:
  virtual UL doGetKey( KeyArg key ) const =0;
  virtual bool doCompareKey( KeyArg key1, KeyArg key2 ) const =0;
  virtual void doFreeAsso( Key &key, Item &item ) {}
};


// Includes:
// #####################Last change: 27/03/98 21h49mn#########################

#include "EBaseHashIteratorT.h"

// 
// #####################Last change: 27/03/98 21h49mn#########################

template<class Key, class Item, class KeyArg, class ItemArg>
EBaseHashT<Key,Item,KeyArg,ItemArg>::EBaseHashT( SL nSize )
{
  EASSERT( nSize > 0 );
  _nNbAsso = 0;
  resize( nSize );
}


template<class Key, class Item, class KeyArg, class ItemArg>
EBaseHashT<Key,Item,KeyArg,ItemArg>::~EBaseHashT()
{
  removeAll();
}


template<class Key, class Item, class KeyArg, class ItemArg>
Item *EBaseHashT<Key,Item,KeyArg,ItemArg>::lookUp( KeyArg key )
{
  SL nIndex = doGetKey( key ) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNext;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Item *EBaseHashT<Key,Item,KeyArg,ItemArg>::lookUp( KeyArg key ) const
{
  SL nIndex = doGetKey( key ) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_item;
    pAsso = pAsso->_pNext;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
const Key *EBaseHashT<Key,Item,KeyArg,ItemArg>::lookUpKey( KeyArg key ) const
{
  SL nIndex = doGetKey( key ) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )
      return &pAsso->_key;
    pAsso = pAsso->_pNext;
  }
  return NULL;
}


template<class Key, class Item, class KeyArg, class ItemArg>
Item& EBaseHashT<Key,Item,KeyArg,ItemArg>::operator[]( KeyArg key )
{
  Item *pItem = lookUp( key );
  EASSERT( pItem );
  return *pItem;
}

/*
template<class Key, class Item, class KeyArg, class ItemArg>
const Item& EBaseHashT<Key,Item,KeyArg,ItemArg>::operator[]( const Key key ) const
{
  const Item *pItem = lookUp( key );
  EASSERT( pItem );
  return *pItem;
}
*/

template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashT<Key,Item,KeyArg,ItemArg>::add( KeyArg key,
                                          ItemArg item )
{
  EASSERT( _aAsso.count() > 0 );

  SL nIndex = doGetKey( key ) % _aAsso.count();

  ELCAsso *pAsso = new ELCAsso();
  pAsso->_key = key;
  pAsso->_item = item;
  pAsso->_pNext = _aAsso[ nIndex ];
  _aAsso[ nIndex ] = pAsso;
  _nNbAsso++;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashT<Key,Item,KeyArg,ItemArg>::remove( KeyArg key )
{
  EASSERT( _aAsso.count() > 0 );

  SL nIndex = doGetKey( key ) % _aAsso.count();
  ELCAsso *pAsso = _aAsso[ nIndex ];
  ELCAsso **pPrevAsso = &_aAsso[ nIndex ];
  while ( pAsso )
  {
    if ( doCompareKey( pAsso->_key, key ) )  // found !
    {
      *pPrevAsso = pAsso->_pNext;
      doFreeAsso( pAsso->_key, pAsso->_item );
      delete pAsso;
      _nNbAsso--;
      return;
    }

    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }
  EASSERT( pAsso );
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashT<Key,Item,KeyArg,ItemArg>::resize( SL nSize )
{
  EDynArray<ELCAsso *> aAsso2;
  SL nCpt;
  ELCAsso *pNull = ENIL;

  aAsso2.resize( nSize );
  aAsso2.fill( &pNull );

  for ( nCpt = 0; nCpt < size(); ++nCpt )
  {
    ELCAsso *pAsso = _aAsso[ nCpt ];
    while ( pAsso )
    {
      ELCAsso *pNextAsso = pAsso->_pNext;
      SL nIndex = doGetKey( pAsso->_key ) % nSize;    // compute new index

      pAsso->_pNext = aAsso2[ nIndex ];              // add to head of lc
      aAsso2[ nIndex ] = pAsso;

      pAsso = pNextAsso;
    }
  }

  _aAsso = aAsso2;
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL EBaseHashT<Key,Item,KeyArg,ItemArg>::size() const
{
  return _aAsso.count();
}


template<class Key, class Item, class KeyArg, class ItemArg>
inline SL EBaseHashT<Key,Item,KeyArg,ItemArg>::count() const
{
  return _nNbAsso;
}


template<class Key, class Item, class KeyArg, class ItemArg>
void EBaseHashT<Key,Item,KeyArg,ItemArg>::removeAll()
{
  SL nIndex;

  for ( nIndex = size() -1; nIndex >= 0; --nIndex )
  {
    ELCAsso *pAsso = _aAsso[ nIndex ];
    while ( pAsso )
    {
      ELCAsso *pNextAsso = pAsso->_pNext;
      doFreeAsso( pAsso->_key, pAsso->_item );
      delete pAsso;
      pAsso = pNextAsso;
    }
    _aAsso[ nIndex ] = ENIL;
  }
  _nNbAsso = 0;
}


template<class Key, class Item, class KeyArg, class ItemArg>
bool 
EBaseHashT<Key,Item,KeyArg,ItemArg>::isEmpty() const
{
  return count() == 0;
}


#endif // !defined(AFX_EBASEHASHT_H__9BDBE5A1_E110_11D2_9222_004005446EE6__INCLUDED_)
