// EBasicStack.h: interface for the EBasicStack class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_EBASICSTACK_H__FEA0CFA8_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
#define AFX_EBASICSTACK_H__FEA0CFA8_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


/* \brief This class represents a generic stack for simple type.
 *
 * @param TElm Type of elements stored in the stack.
 *
 * For pushing, argument is a reference on the element to push.
 * 
 * Element must have a copy operator.
 */
template<class TElm>
class EBasicStack
{
  public:
    /*! Contructs a generic Stack.
     */
    EBasicStack() {};

    /*! Destructor.
     */
    virtual ~EBasicStack() {}

    /*! Tests if the stack is empty.
     * @return \c true if the stack is empty.
     */
    virtual bool isEmpty() const =0;

    /*! Push an element on the stack.
     * @param elm Element to push on the stack.
     */
    virtual void push( TElm elm ) =0;

    /*! Pop an element from the stack.
     *
     * Pop an element from the stack. The Stack must not be empty.
     *
     * @return Poped element.
     * @exception EXCStackEmpty raise if the stack is empty.
     */
    virtual TElm pop() =0;

    /*! Get the top element of the stack.
     * 
     * @return Reference on the top element of the stack. The stack must not be empty.
     * @exception EXCStackEmpty raise if the stack is empty.
     */
    virtual TElm &top() =0;

    /*! Clear the stack.
     *
     * Remove all elements from the stack.
     */
    virtual void clear() =0;
};

#endif // !defined(AFX_EBASICSTACK_H__FEA0CFA8_91FD_11D2_AF1C_AED26EBCA961__INCLUDED_)
