// ###########################################################################
// File: ECallback1RT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK1RT_H__B78AA258_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK1RT_H__B78AA258_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TypeParam1, class ReturnType>
class ECallback1RT : protected EBaseCallback
{
public:
  ECallback1RT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()(TypeParam1 p1) const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb, TypeParam1 p1);

  ECallback1RT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function, class TypeParam1,class ReturnType >
class ECallbackCallerFn1R : public ECallback1RT<TypeParam1, ReturnType>
{
public:
  ECallbackCallerFn1R( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb, TypeParam1 p1 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType, class TypeParam1
         ,class ReturnTypeFn, class TypeParam1Fn>
inline ECallbackCallerFn1R< ReturnTypeFn (*)(TypeParam1Fn)
                                   ,TypeParam1,ReturnType >
eCbCreator( ECallback1RT<TypeParam1, ReturnType> *, 
                         ReturnTypeFn (*fn)(TypeParam1Fn) )
{
  return ECallbackCallerFn1R< ReturnTypeFn (*)(TypeParam1Fn)
                            ,TypeParam1,ReturnType>( fn );
}

template< class Receiver, class Member, class TypeParam1,class ReturnType>
class ECallbackCallerMember1R : public ECallback1RT<TypeParam1, ReturnType>
{
public:
  ECallbackCallerMember1R( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb, TypeParam1 p1 );
};


template< class Receiver, class CallObjType
,class ReturnTypeFn, class TypeParam1Fn
, class TypeParam1,class ReturnType >
inline ECallbackCallerMember1R< Receiver, 
                                       ReturnTypeFn (CallObjType::*)(TypeParam1Fn)
                                       ,TypeParam1,ReturnType >
eCbCreator( ECallback1RT<TypeParam1, ReturnType> *,
            Receiver &objReceiver,
            ReturnTypeFn (CallObjType::* const &fnMemberCallback)(TypeParam1Fn) )
{
  typedef ReturnTypeFn (CallObjType::*MemberCallback)(TypeParam1Fn);

  return ECallbackCallerMember1R< Receiver, MemberCallback
                                ,TypeParam1,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TypeParam1, class ReturnType>
inline 
ECallback1RT<TypeParam1, ReturnType>::operator bool() const
{
  return isValid();
}


template<class TypeParam1, class ReturnType>
inline 
ReturnType
ECallback1RT<TypeParam1, ReturnType>::operator ()(TypeParam1 p1) const
{
  return _fnCaller( *this, p1 );
}


template<class TypeParam1, class ReturnType>
inline
ECallback1RT<TypeParam1, ReturnType>::ECallback1RT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function, class TypeParam1,class ReturnType >
inline
ECallbackCallerFn1R< Function,TypeParam1,ReturnType >::ECallbackCallerFn1R( Function fnCallback ) :
    ECallback1RT<TypeParam1, ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function, class TypeParam1,class ReturnType >
inline ReturnType
ECallbackCallerFn1R< Function,TypeParam1,ReturnType >::caller( const EBaseCallback &cb, TypeParam1 p1 )
{
  return ((Function)cb._fnCallback)(p1);
}


template< class Receiver, class Member, class TypeParam1,class ReturnType >
inline 
ECallbackCallerMember1R<Receiver,Member,TypeParam1,ReturnType>::ECallbackCallerMember1R( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback1RT<TypeParam1, ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member, class TypeParam1,class ReturnType >
inline ReturnType
ECallbackCallerMember1R<Receiver,Member,TypeParam1,ReturnType>::caller( const EBaseCallback &cb, TypeParam1 p1 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)(p1);
}

#endif // !defined(AFX_ECALLBACK1RT_H__B78AA258_D476_11D2_BD55_0040053AD648__INCLUDED_)
