// ###########################################################################
// File: ECallback1T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK1T_H__B78AA257_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK1T_H__B78AA257_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TypeParam1>
class ECallback1T : protected EBaseCallback
{
public:
  ECallback1T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TypeParam1 p1) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TypeParam1 p1);

  ECallback1T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function, class TypeParam1 >
class ECallbackCallerFn1 : public ECallback1T<TypeParam1>
{
public:
  ECallbackCallerFn1( Function fnCallback );

  static void caller( const EBaseCallback &cb, TypeParam1 p1 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType, class TypeParam1
         ,class TypeParam1Fn>
inline ECallbackCallerFn1< ReturnType (*)(TypeParam1Fn)
                                   ,TypeParam1 >
eCbCreator( ECallback1T<TypeParam1> *, 
                         ReturnType (*fn)(TypeParam1Fn) )
{
  return ECallbackCallerFn1< ReturnType (*)(TypeParam1Fn)
                            ,TypeParam1>( fn );
}

template< class Receiver, class Member, class TypeParam1>
class ECallbackCallerMember1 : public ECallback1T<TypeParam1>
{
public:
  ECallbackCallerMember1( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TypeParam1 p1 );
};


template< class Receiver, class CallObjType
,class TypeParam1Fn
, class TypeParam1,class ReturnType >
inline ECallbackCallerMember1< Receiver, 
                                       ReturnType (CallObjType::*)(TypeParam1Fn)
                                       ,TypeParam1 >
eCbCreator( ECallback1T<TypeParam1> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TypeParam1Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TypeParam1Fn);

  return ECallbackCallerMember1< Receiver, MemberCallback
                                ,TypeParam1>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TypeParam1>
inline 
ECallback1T<TypeParam1>::operator bool() const
{
  return isValid();
}


template<class TypeParam1>
inline 
void
ECallback1T<TypeParam1>::operator ()(TypeParam1 p1) const
{
   _fnCaller( *this, p1 );
}


template<class TypeParam1>
inline
ECallback1T<TypeParam1>::ECallback1T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function, class TypeParam1 >
inline
ECallbackCallerFn1< Function,TypeParam1 >::ECallbackCallerFn1( Function fnCallback ) :
    ECallback1T<TypeParam1>( caller, NULL, fnCallback, 0 )
{
}


template< class Function, class TypeParam1 >
inline void
ECallbackCallerFn1< Function,TypeParam1 >::caller( const EBaseCallback &cb, TypeParam1 p1 )
{
   ((Function)cb._fnCallback)(p1);
}


template< class Receiver, class Member, class TypeParam1 >
inline 
ECallbackCallerMember1<Receiver,Member,TypeParam1>::ECallbackCallerMember1( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback1T<TypeParam1>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member, class TypeParam1 >
inline void
ECallbackCallerMember1<Receiver,Member,TypeParam1>::caller( const EBaseCallback &cb, TypeParam1 p1 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1);
}


#endif // !defined(AFX_ECALLBACK1T_H__B78AA257_D476_11D2_BD55_0040053AD648__INCLUDED_)
