// ###########################################################################
// File: ECallback4RT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK4RT_H__B78AA25F_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK4RT_H__B78AA25F_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class TP4,class ReturnType>
class ECallback4RT : protected EBaseCallback
{
public:
  ECallback4RT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4) const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4);

  ECallback4RT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class TP4,class ReturnType >
class ECallbackCallerFn4RT : public ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>
{
public:
  ECallbackCallerFn4RT( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3,class TP4
         ,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn>
inline ECallbackCallerFn4RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                                   ,TP1,TP2,TP3,TP4,ReturnType >
eCbCreator( ECallback4RT<TP1,TP2,TP3,TP4,ReturnType> *, 
                         RTpFn (*fn)(TP1Fn,TP2Fn,TP3Fn,TP4Fn) )
{
  return ECallbackCallerFn4RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                            ,TP1,TP2,TP3,TP4,ReturnType>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class ReturnType>
class ECallbackCallerMember4RT : public ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>
{
public:
  ECallbackCallerMember4RT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4 );
};


template< class Receiver, class CallObjType
,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn
,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline ECallbackCallerMember4RT< Receiver, 
                                       RTpFn (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                                       ,TP1,TP2,TP3,TP4,ReturnType >
eCbCreator( ECallback4RT<TP1,TP2,TP3,TP4,ReturnType> *,
            Receiver &objReceiver,
            RTpFn (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn) )
{
  typedef RTpFn (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn);

  return ECallbackCallerMember4RT< Receiver, MemberCallback
                                ,TP1,TP2,TP3,TP4,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class TP4,class ReturnType>
inline 
ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class TP4,class ReturnType>
inline 
ReturnType
ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>::operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4) const
{
  return _fnCaller( *this,p1,p2,p3,p4 );
}


template<class TP1,class TP2,class TP3,class TP4,class ReturnType>
inline
ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>::ECallback4RT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline
ECallbackCallerFn4RT< Function,TP1,TP2,TP3,TP4,ReturnType >::ECallbackCallerFn4RT( Function fnCallback ) :
    ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline ReturnType
ECallbackCallerFn4RT< Function,TP1,TP2,TP3,TP4,ReturnType >::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4 )
{
  return ((Function)cb._fnCallback)(p1,p2,p3,p4);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline 
ECallbackCallerMember4RT<Receiver,Member,TP1,TP2,TP3,TP4,ReturnType>::ECallbackCallerMember4RT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback4RT<TP1,TP2,TP3,TP4,ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline ReturnType
ECallbackCallerMember4RT<Receiver,Member,TP1,TP2,TP3,TP4,ReturnType>::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)(p1,p2,p3,p4);
}


#endif // !defined(AFX_ECALLBACK4RT_H__B78AA25F_D476_11D2_BD55_0040053AD648__INCLUDED_)
