// ###########################################################################
// File: ECallback4T.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK4T_H__B78AA25E_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK4T_H__B78AA25E_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class TP4>
class ECallback4T : protected EBaseCallback
{
public:
  ECallback4T() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  void operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4) const;


  typedef void (*ECallerFn)(const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4);

  ECallback4T( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class TP4 >
class ECallbackCallerFn4T : public ECallback4T<TP1,TP2,TP3,TP4>
{
public:
  ECallbackCallerFn4T( Function fnCallback );

  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3,class TP4
         ,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn>
inline ECallbackCallerFn4T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                                   ,TP1,TP2,TP3,TP4 >
eCbCreator( ECallback4T<TP1,TP2,TP3,TP4> *, 
                         ReturnType (*fn)(TP1Fn,TP2Fn,TP3Fn,TP4Fn) )
{
  return ECallbackCallerFn4T< ReturnType (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                            ,TP1,TP2,TP3,TP4>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4>
class ECallbackCallerMember4T : public ECallback4T<TP1,TP2,TP3,TP4>
{
public:
  ECallbackCallerMember4T( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static void caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4 );
};


template< class Receiver, class CallObjType
,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn
,class TP1,class TP2,class TP3,class TP4,class ReturnType >
inline ECallbackCallerMember4T< Receiver, 
                                       ReturnType (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn)
                                       ,TP1,TP2,TP3,TP4 >
eCbCreator( ECallback4T<TP1,TP2,TP3,TP4> *,
            Receiver &objReceiver,
            ReturnType (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn) )
{
  typedef ReturnType (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn);

  return ECallbackCallerMember4T< Receiver, MemberCallback
                                ,TP1,TP2,TP3,TP4>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class TP4>
inline 
ECallback4T<TP1,TP2,TP3,TP4>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class TP4>
inline 
void
ECallback4T<TP1,TP2,TP3,TP4>::operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4) const
{
   _fnCaller( *this, p1,p2,p3,p4 );
}


template<class TP1,class TP2,class TP3,class TP4>
inline
ECallback4T<TP1,TP2,TP3,TP4>::ECallback4T( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4 >
inline
ECallbackCallerFn4T< Function,TP1,TP2,TP3,TP4 >::ECallbackCallerFn4T( Function fnCallback ) :
    ECallback4T<TP1,TP2,TP3,TP4>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4 >
inline void
ECallbackCallerFn4T< Function,TP1,TP2,TP3,TP4 >::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4 )
{
   ((Function)cb._fnCallback)(p1,p2,p3,p4);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4 >
inline 
ECallbackCallerMember4T<Receiver,Member,TP1,TP2,TP3,TP4>::ECallbackCallerMember4T( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback4T<TP1,TP2,TP3,TP4>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4 >
inline void
ECallbackCallerMember4T<Receiver,Member,TP1,TP2,TP3,TP4>::caller( const EBaseCallback &cb, TP1 p1,TP2 p2,TP3 p3,TP4 p4 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

   (objReceiver.*fnMember)(p1,p2,p3,p4);
}

#endif // !defined(AFX_ECALLBACK4T_H__B78AA25E_D476_11D2_BD55_0040053AD648__INCLUDED_)
