// ###########################################################################
// File: ECallback5RT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACK5RT_H__B78AA260_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACK5RT_H__B78AA260_D476_11D2_BD55_0040053AD648__INCLUDED_

#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType>
class ECallback5RT : protected EBaseCallback
{
public:
  ECallback5RT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5) const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5);

  ECallback5RT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
class ECallbackCallerFn5RT : public ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>
{
public:
  ECallbackCallerFn5RT( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType,class TP1,class TP2,class TP3,class TP4,class TP5
         ,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn>
inline ECallbackCallerFn5RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                                   ,TP1,TP2,TP3,TP4,TP5,ReturnType >
eCbCreator( ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType> *, 
                         RTpFn (*fn)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn) )
{
  return ECallbackCallerFn5RT< RTpFn (*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                            ,TP1,TP2,TP3,TP4,TP5,ReturnType>( fn );
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType>
class ECallbackCallerMember5RT : public ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>
{
public:
  ECallbackCallerMember5RT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 );
};


template< class Receiver, class CallObjType
,class RTpFn,class TP1Fn,class TP2Fn,class TP3Fn,class TP4Fn,class TP5Fn
,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline ECallbackCallerMember5RT< Receiver, 
                                       RTpFn (CallObjType::*)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn)
                                       ,TP1,TP2,TP3,TP4,TP5,ReturnType >
eCbCreator( ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType> *,
            Receiver &objReceiver,
            RTpFn (CallObjType::* const &fnMemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn) )
{
  typedef RTpFn (CallObjType::*MemberCallback)(TP1Fn,TP2Fn,TP3Fn,TP4Fn,TP5Fn);

  return ECallbackCallerMember5RT< Receiver, MemberCallback
                                ,TP1,TP2,TP3,TP4,TP5,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType>
inline 
ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>::operator bool() const
{
  return isValid();
}


template<class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType>
inline 
ReturnType
ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>::operator ()(TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5) const
{
  return _fnCaller( *this,p1,p2,p3,p4,p5 );
}


template<class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType>
inline
ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>::ECallback5RT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline
ECallbackCallerFn5RT< Function,TP1,TP2,TP3,TP4,TP5,ReturnType >::ECallbackCallerFn5RT( Function fnCallback ) :
    ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline ReturnType
ECallbackCallerFn5RT< Function,TP1,TP2,TP3,TP4,TP5,ReturnType >::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 )
{
  return ((Function)cb._fnCallback)(p1,p2,p3,p4,p5);
}


template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline 
ECallbackCallerMember5RT<Receiver,Member,TP1,TP2,TP3,TP4,TP5,ReturnType>::ECallbackCallerMember5RT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallback5RT<TP1,TP2,TP3,TP4,TP5,ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member,class TP1,class TP2,class TP3,class TP4,class TP5,class ReturnType >
inline ReturnType
ECallbackCallerMember5RT<Receiver,Member,TP1,TP2,TP3,TP4,TP5,ReturnType>::caller( const EBaseCallback &cb,TP1 p1,TP2 p2,TP3 p3,TP4 p4,TP5 p5 )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)(p1,p2,p3,p4,p5);
}

#endif // !defined(AFX_ECALLBACK5RT_H__B78AA260_D476_11D2_BD55_0040053AD648__INCLUDED_)
