// ###########################################################################
// File: ECallbackRT.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################
#if !defined(AFX_ECALLBACKRT_H__B78AA256_D476_11D2_BD55_0040053AD648__INCLUDED_)
#define AFX_ECALLBACKRT_H__B78AA256_D476_11D2_BD55_0040053AD648__INCLUDED_


#include "EBaseCallback.h"

/*! 
 * \brief This class provides a callback without parameters and return value.
 */
template<class ReturnType>
class ECallbackRT : protected EBaseCallback
{
public:
  ECallbackRT() {}

  /*! Tests if the callback has been set.
   * @return \c true if the callback has been set, otherwise \c false.
   */
  operator bool() const;

  /*! Calls the callback.
   */
  ReturnType operator ()() const;


  typedef ReturnType (*ECallerFn)(const EBaseCallback &cb);

  ECallbackRT( ECallerFn fnCaller,
             void *pObjReceiver,
             const void *fnMember,
             size_t nSizeFnMember );

private:
  ECallerFn _fnCaller;
};



template< class Function, class ReturnType >
class ECallbackCallerFnRT : public ECallbackRT<ReturnType>
{
public:
  ECallbackCallerFnRT( Function fnCallback );

  static ReturnType caller( const EBaseCallback &cb );
};

/*! \brief This class provides a Prototype for static method callback.
 */
template<class ReturnType
         , class ReturnTypeFn>
inline ECallbackCallerFnRT< ReturnTypeFn (*)()
                                   ,ReturnType >
eCbCreator( ECallbackRT<ReturnType> *, 
                         ReturnTypeFn (*fn)() )
{
  return ECallbackCallerFnRT< ReturnTypeFn (*)()
                            ,ReturnType>( fn );
}

template< class Receiver, class Member, class ReturnType>
class ECallbackCallerMemberRT : public ECallbackRT<ReturnType>
{
public:
  ECallbackCallerMemberRT( Receiver *pObjReceiver, 
                                  const Member &fnMember );


  static ReturnType caller( const EBaseCallback &cb );
};


template< class Receiver, class CallObjType
, class ReturnTypeFn
,class ReturnType >
inline ECallbackCallerMemberRT< Receiver, 
                                       ReturnTypeFn (CallObjType::*)()
                                       ,ReturnType >
eCbCreator( ECallbackRT<ReturnType> *,
            Receiver &objReceiver,
            ReturnTypeFn (CallObjType::* const &fnMemberCallback)() )
{
  typedef ReturnTypeFn (CallObjType::*MemberCallback)();

  return ECallbackCallerMemberRT< Receiver, MemberCallback
                                ,ReturnType>( &objReceiver,
                                                            fnMemberCallback );
}






// ###################################################################
// Inline methods:
// ###################################################################


template<class ReturnType>
inline 
ECallbackRT<ReturnType>::operator bool() const
{
  return isValid();
}


template<class ReturnType>
inline 
ReturnType
ECallbackRT<ReturnType>::operator ()() const
{
  return _fnCaller( *this );
}


template<class ReturnType>
inline
ECallbackRT<ReturnType>::ECallbackRT( ECallerFn fnCaller,
                      void *pObjReceiver,
                      const void *fnMember,
                      size_t nSizeFnMember ) :
    EBaseCallback( pObjReceiver,
                   fnMember,
                   nSizeFnMember ),
    _fnCaller( fnCaller )
{
}


template< class Function, class ReturnType >
inline
ECallbackCallerFnRT< Function,ReturnType >::ECallbackCallerFnRT( Function fnCallback ) :
    ECallbackRT<ReturnType>( caller, NULL, fnCallback, 0 )
{
}


template< class Function, class ReturnType >
inline ReturnType
ECallbackCallerFnRT< Function,ReturnType >::caller( const EBaseCallback &cb )
{
  return ((Function)cb._fnCallback)();
}


template< class Receiver, class Member, class ReturnType >
inline 
ECallbackCallerMemberRT<Receiver,Member,ReturnType>::ECallbackCallerMemberRT( Receiver *pObjReceiver, 
                                                               const Member &fnMember ) :
    ECallbackRT<ReturnType>( caller, pObjReceiver, &fnMember, sizeof( fnMember ) )
{
}

template< class Receiver, class Member, class ReturnType >
inline ReturnType
ECallbackCallerMemberRT<Receiver,Member,ReturnType>::caller( const EBaseCallback &cb )
{
  Receiver &objReceiver = *(Receiver *)cb._pObjReceiver;
  Member &fnMember = *(Member *)(void *)cb._fnMemberCallback;

  return (objReceiver.*fnMember)();
}


#endif // !defined(AFX_ECALLBACKRT_H__B78AA256_D476_11D2_BD55_0040053AD648__INCLUDED_)

