// ###########################################################################
// File: EHashValBoolIt.h
// (c)Copyright 03/1999, Gaia Creation, Baptiste Lepilleur
// ###########################################################################

#if !defined(AFX_EHASHVALBOOLIT_H__FDCB40E3_DFC8_11D2_BD6F_0040053AD648__INCLUDED_)
#define AFX_EHASHVALBOOLIT_H__FDCB40E3_DFC8_11D2_BD6F_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

// Includes:
// #####################Last change: 28/03/98 20h01mn#########################
#include "EIteratorT.h"

// Constantes:
// #####################Last change: 28/03/98 20h01mn#########################


// Macros:
// #####################Last change: 28/03/98 20h01mn#########################


// Types:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class KeyArg>
class EHashValBoolIteratorT : public EIteratorT<bool>
{
public:
  EHashValBoolIteratorT( const EHashValBoolT<Key,KeyArg> &hashTable );
  EHashValBoolIteratorT();

  void attach(  const EHashValBoolT<Key,KeyArg> &hashTable );

  /** Set the iterator at the beginning of the table.
   * If there is no element in the table then isEnd() will return true.
   */
  void begin();

  /** Test if the iterator is beyond the last association.
   */
  bool isEnd() const;

  /** Remove the association at the current position of the iterator.
   */
  void remove();

  /** Move the iterator to the next association.
   */
  void next();

  /** Get the key at the current position of the iterator.
   * @return key at the current position of the iterator.
   */
  const Key& key() const;

  /*! Gets the item at the current position of the iterator.
   * @return Item at the current position of the iterator.
   */
  bool& item();

  /*! Gets the item at the current position of the iterator.
   * @return Item at the current position of the iterator.
   */
  const bool& item() const;

  /*! Get the number of items.
   * @return Number of items.
   */
  SL count() const;

private:
  /** Search the next index with an association.
   * The search begins at the index _nIndex +1.
   */
  void searchNextIndex();

  /// Hash table the iterator operate on.
  const EHashValBoolT<Key,KeyArg> *_pHashTable;

  /// Current index in association table.
  SL _nIndex;

  /// Current Association for the list at nIndex.
  EHashValBoolT<Key,KeyArg>::ELCAsso *_pAsso;

  bool _bDummy;
};


// Template implementation:
// #####################Last change: 28/03/98 20h01mn#########################

template<class Key, class KeyArg>
EHashValBoolIteratorT<Key,KeyArg>::EHashValBoolIteratorT( 
    const EHashValBoolT<Key,KeyArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}


template<class Key, class KeyArg>
EHashValBoolIteratorT<Key,KeyArg>::EHashValBoolIteratorT()
{
  _pHashTable = NULL;
  _nIndex = 0;
  _pAsso = ENIL;
}


template<class Key, class KeyArg>
void EHashValBoolIteratorT<Key,KeyArg>::attach(  
    const EHashValBoolT<Key,KeyArg> &hashTable )
{
  _pHashTable = &hashTable;
  _nIndex = 0;
  _pAsso = ENIL;
  begin();
}

template<class Key, class KeyArg>
void EHashValBoolIteratorT<Key,KeyArg>::begin()
{
  EASSERT( _pHashTable );
  _nIndex = -1;
  searchNextIndex();
}

template<class Key, class KeyArg>
bool EHashValBoolIteratorT<Key,KeyArg>::isEnd() const
{
  EASSERT( _pHashTable );
  return _pAsso == ENIL;
}


template<class Key, class KeyArg>
void EHashValBoolIteratorT<Key,KeyArg>::remove()
{
  EASSERT( _pHashTable );
  if ( isEnd() )
    return;

  EHashVal<Key,Item,KeyArg,ItemArg>::ELCAsso *pAsso =
      _pHashTable->_aAsso[ _nIndex ];
  EHashVal<Key,Item,KeyArg,ItemArg>::ELCAsso **pPrevAsso =
      &_pHashTable->_aAsso[ _nIndex ];

  while ( pAsso != _pAsso )     // search previous association
  {
    EASSERT( pAsso );
    pPrevAsso = &pAsso->_pNext;
    pAsso = pAsso->_pNext;
  }

  _pHashTable->freeAsso( _pAsso->_key, _pAsso->_item );

  *pPrevAsso = _pAsso->_pNext;       // unchain
  delete _pAsso;
  _pAsso = *pPrevAsso;
  _pHashTable->_nNbAsso--;

  if ( !_pAsso )                  // if no more in the list for this index
  {                               // then search next.
    searchNextIndex();
  }
}


template<class Key, class KeyArg>
void EHashValBoolIteratorT<Key,KeyArg>::next()
{
  EASSERT( _pHashTable );
  if ( _pAsso )
  {
    _pAsso = _pAsso->_pNext;
    if ( !_pAsso )          // end of current index reached ?
      searchNextIndex();
  }
}


template<class Key, class KeyArg>
void EHashValBoolIteratorT<Key,KeyArg>::searchNextIndex()
{
  EASSERT( _pHashTable );
  SL nSize = _pHashTable->size();
  _nIndex++;
  for ( ;_nIndex < nSize; _nIndex++ )
  {
    if ( _pHashTable->_aAsso[ _nIndex ] )
    {
      _pAsso = _pHashTable->_aAsso[ _nIndex ];
      return;
    }
  }
  _pAsso = ENIL;
}


template<class Key, class KeyArg>
const Key& EHashValBoolIteratorT<Key,KeyArg>::key() const
{
  EASSERT( _pHashTable );
  EASSERT( _pAsso );
  return _pAsso->_key;
}


template<class Key, class KeyArg>
inline SL 
EHashValBoolIteratorT<Key,KeyArg>::count() const
{
  EASSERT( _pHashTable );
  return _pHashTable->count();
}


template<class Key, class KeyArg>
bool& 
EHashValBoolIteratorT<Key,KeyArg>::item()
{
  _bDummy = true;
  return _bDummy;
}


template<class Key, class KeyArg>
const bool& 
EHashValBoolIteratorT<Key,KeyArg>::item() const
{
  ((EHashValBoolIteratorT<Key,KeyArg> *)this)->_bDummy = true;
  return _bDummy;
}


#endif // !defined(AFX_EHASHVALBOOLIT_H__FDCB40E3_DFC8_11D2_BD6F_0040053AD648__INCLUDED_)
