// ESize.h: interface for the ESize class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESIZE_H__180BBAEC_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
#define AFX_ESIZE_H__180BBAEC_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "EVector2.h"

template<class EGmt>
class ESizeT
{
public:
	
  /*! Constructs a size.
   * @param fWidth Width.
   * @param fHeight Height.
   */
  ESizeT( EGmt fWidth =0, 
           EGmt fHeight =0 );
	
  /*! Constructs a size using its diagonal
   * @param v2Diagonal New diagonal size.
   */
  ESizeT( const EVector2 &v2Diagonal );

	/*! Destructor.
   */
  virtual ~ESizeT() {};

  /*! Set the size.
   * @param fWidth Width.
   * @param fHeight Height.
   */
  void setSize( EGmt fWidth, 
                EGmt fHeight );

  /*! Set the size.
   * @param v2Diagonal New diagonal size.
   */
  void setSize( const EVector2 &v2Diagonal );

  /*! Set the width.
   * @param fWidth Width.
   */
  void setWidth( EGmt fWidth );


  /*! Set the height.
   * @param fHeight Height.
   */
  void setHeight( EGmt fHeight );

  /*! Set size to null size.
   */
  void setNull();

  /*! Get the width.
   * @return Width.
   */
  EGmt width() const;

  /*! Get the height.
   * @return Height.
   */
  EGmt height() const;

  /*! Scales. 
   * @param fScale Scale factor.
   */
  void scale( float fScale );

  /*! Get the diagonal vector.
   * @return Vector equal to (Width, Height).
   */
  EVector2 diagonal() const;

  /*! Enlarge the diagonal.
   * @param v2IncDiagonal Vector to add to the diagonal.
   */
  void addToDiagonal( const EVector2 &v2IncDiagonal );

private:
  EGmt _fWidth;
  EGmt _fHeight;
};


typedef ESizeT<float> ESize;
typedef ESizeT<SL> ESizeI;


template<class EGmt>
inline 
ESizeT<EGmt>::ESizeT( EGmt fWidth, 
                        EGmt fHeight )
{
  _fWidth = fWidth;
  _fHeight = fHeight;
}


template<class EGmt>
inline
ESizeT<EGmt>::ESizeT( const EVector2 &v2Diagonal )
{
  setSize( v2Diagonal );
}


template<class EGmt>
inline void 
ESizeT<EGmt>::setSize( EGmt fWidth, 
                        EGmt fHeight )
{
  _fWidth = fWidth;
  _fHeight = fHeight;
}


template<class EGmt>
inline void 
ESizeT<EGmt>::setSize( const EVector2 &v2Diagonal )
{
  setSize( v2Diagonal.x(), v2Diagonal.y() );
}


template<class EGmt>
inline void 
ESizeT<EGmt>::setWidth( EGmt fWidth )
{
  _fWidth = fWidth;
}


template<class EGmt>
inline void 
ESizeT<EGmt>::setHeight( EGmt fHeight )
{
  _fHeight = fHeight;
}


template<class EGmt>
inline EGmt 
ESizeT<EGmt>::width() const
{
  return _fWidth;
}


template<class EGmt>
inline EGmt 
ESizeT<EGmt>::height() const
{
  return _fHeight;
}


template<class EGmt>
inline void 
ESizeT<EGmt>::scale( float fScale )
{
  _fWidth *= fScale;
  _fHeight *= fScale;
}


template<class EGmt>
inline EVector2 
ESizeT<EGmt>::diagonal() const
{
  return EVector2( _fWidth, _fHeight );
}


template<class EGmt>
void 
ESizeT<EGmt>::setNull()
{
  _fWidth =0;
  _fHeight =0;
}


template<class EGmt>
void 
ESizeT<EGmt>::addToDiagonal( const EVector2 &v2IncDiagonal )
{
  _fWidth = EMAX( 0, _fWidth + v2IncDiagonal.x() );
  _fHeight = EMAX( 0, _fHeight + v2IncDiagonal.y() );
}

/*
template<class EGmt>
EDataStream &operator << ( EDataStream &s, const ESize<EGmt> &size )
{
}


template<class EGmt>
EDataStream &operator >> ( EDataStream &s, ESize<EGmt> &size )
{
}
*/
#endif // !defined(AFX_ESIZE_H__180BBAEC_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
