// EStoreObjectReader.cpp: implementation of the EStoreObjectReader class.
//
//////////////////////////////////////////////////////////////////////

#include "usuel.hpp"
#include "EString.hpp"
#include "EIoDevic.hpp"
#include "EDataStr.hpp"
#include "EStoreOtReader.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EStoreOtReader::EStoreOtReader() :
    _pstreamData( NULL )
{

}


EStoreOtReader::~EStoreOtReader()
{

}


void 
EStoreOtReader::read( EDataStream &streamIn )
{
  // Initialize reading
  _pstreamData = &streamIn;
  _nLineNo =1;
  _bIsCr =false;
  _bIsEndOfStream =false;  

  _context.setDataStream( _pstreamData );

  doRead();
}


void 
EStoreOtReader::readLine( EString &strLine )
{
  _context.setLineNo( _nLineNo );
  strLine.setNull();

  if ( _bIsEndOfStream )
    return;
  
  ETRY
  {
    UB ubChar;
    while ( (ubChar = readByte()) != '\n' )
    {
      strLine += ubChar;
    }
  }
  ECATCH( EXCIODevice, exc )
  {
    if ( exc.cause() == EXCIODevice::endOfFile )
    {
      _bIsEndOfStream =true;
    }
    else
    {
      ETHROW_AGAIN;
    }
  }
}


EStoreOtReader::TokenType
EStoreOtReader::readToken()
{
  EString strLine;

  do
  {
    readLine( strLine );
    strLine.trimSpaces();
  }
  while ( strLine.isNull()  &&
          !_bIsEndOfStream );   // skip empty line

  if ( strLine.isNull() )
    return _bIsEndOfStream;

  TCHAR tcHead = strLine[ 0 ];

  if ( tcHead == _T( '(' ) )
  {
    return makeTokenObject( strLine );
  }
  else if ( tcHead == _T( ')' ) )
  {
    if ( strLine.length() > 1 )     // syntax error
      return makeTokenError( errSyntax );
    return tktpObjectClose;
  }
  else
  {
    return makeTokenPrp( strLine );
  }
}


EStoreOtReader::TokenType 
EStoreOtReader::makeTokenError( Error err )
{
  _errToken = err;
  return tktpError;
}


EStoreOtReader::TokenType 
EStoreOtReader::makeTokenObject( EString &strLine )
{
  // skip white following '('
  SL nPos;
  for ( nPos =1; nPos < strLine.length(); ++nPos )
  {
    if ( strLine[ nPos ] > 32 )
      break;
  }

  if ( nPos >= strLine.length() )                     // error 
    return makeTokenError( errExpectedObjectType );

  // extract object type
  strLine.remove( 0, nPos );
  EStoreObjectType esotpObject = def().objectType( strLine );
  if ( esotpObject == esotpNone )
    return makeTokenError( errBadObjectType );

  _esotpToken = esotpObject;
  return tktpObject;
}


EStoreOtReader::TokenType 
EStoreOtReader::makeTokenPrp( EString &strLine )
{
  // a propert as the following format:
  // PropertyName = PropertyValue

  // extract the name of the property.
  SL nPosEqual = strLine.pos( _T( '=' ) );
  if ( nPosEqual < 0 )
  {
    _errToken = errExpectedPrpEqual;
    return tktpError;
  }

  EString strPrpName = strLine.left( nPosEqual );
  strPrpName.trimRight();

  EStorePrpType esptpPrp = def().prpType( strPrpName );
  if ( esptpPrp == esptpNone )
  {
    _errToken = errBadPrpType;
    return tktpError;
  }

  // extract the property value
    // search the position of the first char of the value.
  SL nPosPrp ;
  for ( nPosPrp = nPosEqual+1; nPosPrp < strLine.length(); ++nPosPrp )
  {
    if ( strLine[ nPosPrp ] > 32 )
      break;
  }

    // get the value.
  EString strPrpValue = strLine.mid( nPosPrp, -1 );
  if ( strPrpValue.isNull() )
  {
    _errToken = errExpectedPrpValue;
    return tktpError;
  }

  EStoreOtPrp prp( esptpPrp, strPrpValue, false );
  if ( !prp.isValid() )   // invalid property definition !
  {
    _errToken = errBadPrpValue;
    return tktpError;
  }

  // check value type
  const EStoreOtDef::PrpTypeInfo &prpInfo = def().prpTypeInfo( strPrpName );
  if ( prp.valueType() != prpInfo._pvtpPrp )
  {
    _errToken = errBadPrpValueType;
    return tktpError;
  }

  _prpToken = prp;

  return tktpProperty;
}
