// ESymbolHolderT.h: interface for the ESymbolHolderT class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_ESYMBOLHOLDERT_H__992AFCEC_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
#define AFX_ESYMBOLHOLDERT_H__992AFCEC_3D03_11D3_BE1D_0040053AD648__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

/* History:
 * 18/09/1999: - Creation.
 *
 * To do:
 * - When Ansi/Unicode version of EString...
 */


#include "EHashSetT.h"

/*! \class ESymbolHolderT<String,StringKeyProvider>
 * \include "ESymbolHolderT.h"
 * \brief Template to store a collection of symbol.
 *
 * @param String Character type for symbols.
 * @param StringKeyProvider Hash Key Provider for symbol string.
 *                          Should be EStringKeyProviderT<String> or
 *                                    EStringNcsKeyProviderT<String> for non case sensitive.
 */
template< class String,
          class StringKeyProvider >
class ESymbolHolderT  
{
public:
  typedef String ThisString;
  typedef EHashSetT<String,const String,StringKeyProvider> EHashSymbol;

  /*! Default constructor.
   */
  ESymbolHolderT() {}

  /*! Destructor.
   */
  virtual ~ESymbolHolderT() {}

  /*! Build a case-sensitive symbol.
   *
   * Retreive or build a symbol. The reference that is returned is guaranted to
   * exist as long as the symbol is in the symbol holder.
   *
   * @param cszSymbol Symbol name.
   * @return Reference on the symbol.
   */
  const ThisString &symbol( const String &strSymbol );

private:
  /*! Set that hold the symbols.
   */
  EHashSymbol _hashSymbol;
};


typedef ESymbolHolderT<EAString, EStringKeyProviderT<EAString> > EASymbolHolder;
typedef ESymbolHolderT<EAString, EStringNcsKeyProviderT<EAString> > EASymbolNcsHolder;
typedef ESymbolHolderT<EWString, EStringKeyProviderT<EWString> > EWSymbolHolder;
typedef ESymbolHolderT<EWString, EStringNcsKeyProviderT<EWString> > EWSymbolNcsHolder;
#ifdef UNICODE
  typedef EWSymbolHolder ESymbolHolder;
  typedef EWSymbolNcsHolder ESymbolNcsHolder;
#else
  typedef EASymbolHolder ESymbolHolder;
  typedef EASymbolNcsHolder ESymbolNcsHolder;
#endif

// Template implementation ESymbolHolderT<String,StringKeyProvider>
// -------------------------------------------------------------------

template<class String, class StringKeyProvider>
const String &
ESymbolHolderT<String,StringKeyProvider>::symbol( const String &strSymbol )
{
  const ThisString *pstrSymbol = _hashSymbol.lookUp( strSymbol );
  if ( !pstrSymbol )   // Symbol does not exist !
  {
    _hashSymbol.add( strSymbol );
    return *_hashSymbol.lookUp( strSymbol );
  }
  return *pstrSymbol;
}


#endif // !defined(AFX_ESYMBOLHOLDERT_H__992AFCEC_3D03_11D3_BE1D_0040053AD648__INCLUDED_)
