// ###########################################################################
// File: EBITSTRM.HPP
// (c)Copyright 04/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################


#ifndef EBITSTRM_HPP_INCLUDED


// Includes:
// #####################Last change: 04/04/98 17h48mn#########################

#include "estream.hpp"
#include "ebitarr.hpp"

// Constantes:
// #####################Last change: 04/04/98 17h48mn#########################


// Macros:
// #####################Last change: 04/04/98 17h48mn#########################


// Types:
// #####################Last change: 04/04/98 17h48mn#########################

class EBitStream : public EStream
{
public:
  /** Constructs a bit stream for device \a device.
    * @param pDevice device used by the data stream.
    */
  EBitStream( EIODevice *pDevice );

  /// Default constructor.
  EBitStream();

  /// Destructor.
  ~EBitStream();

  /** Set device used by the bit stream.
    * Set the device used by the bit stream.
    * @param pDevice device used by the bit stream.
    */
  void setDevice( EIODevice *pDevice );

  /** Read a bit from the stream.
   * @return true if the bit value is 1, false if it is 0.
   */
  bool getBit();

  /** Write a bit to the stream.
   * @param bBit if it is true then a bit of value 1 is written, 0 otherwise.
   */
  void putBit( bool bBit =1);

  /** Read a block.
    * @param pData data to read.
    * @param nLen length of data to read in byte.
    */
  void readBlock( void *pData, SL nLen );

  /** Write a block.
    * @param pData data to write.
    * @param nLen length of data to write in byte.
    */
  void writeBlock( const void *pData, SL nLen );

  /** Get a byte from the stream.
    * @return byte read.
    */
  UB getByte();

  /** Write a byte in the stream.
    * @param v byte to write.
    */
  void putByte( UB v);

  EBitStream& operator >>(SB& v);
  EBitStream& operator >>(SW& v);
  EBitStream& operator >>(SL& v);
  EBitStream& operator >>(UB& v);
  EBitStream& operator >>(UW& v);
  EBitStream& operator >>(UL& v);
  EBitStream& operator >>(float& v);
  EBitStream& operator >>(double& v);
  EBitStream& operator >>(char *& v);

  EBitStream& operator <<(SB v);
  EBitStream& operator <<(SW v);
  EBitStream& operator <<(SL v);
  EBitStream& operator <<(UB v);
  EBitStream& operator <<(UW v);
  EBitStream& operator <<(UL v);
  EBitStream& operator <<(float v);
  EBitStream& operator <<(double v);
  EBitStream& operator <<(const char * v);

  EBitStream& operator <<( const EBitArray& aBits );

  /// Test if the current mode is reading.
  bool isReadMode() const;
private:
  /// Initialize members.
  void init();

  /// Write the last bit if needed.
  void flush();

  /// Get a new byte if the shift is beyond 8.
  void readFlush();

  /// Write the current byte and reset the byte if shift is beyond 8.
  void writeFlush();

  /// Current bit on the current byte.
  UL _nShift;

  /// Current byte value.
  UB _nByte;

  /// Read Mode.
  bool _bReadMode;
};

// inlines:
// #####################Last change: 04/04/98 17h48mn#########################

/// Get a new byte if the shift is beyond 8.
inline void EBitStream::readFlush()
{
  EASSERT( device() );
  if ( _nShift == 8 )
  {
    _nByte = device()->getByte();
    _nShift = 0;
  }
}

/// Write the current byte and reset the byte if shift is beyond 8.
inline void EBitStream::writeFlush()
{
  EASSERT( device() );
  if ( _nShift == 8 )
  {
    device()->putByte( _nByte );
    _nShift = 0;
    _nByte = 0;
  }
}

/// Test if the current mode is reading.
inline bool EBitStream::isReadMode() const
{
  EASSERT( device() );
  return _bReadMode;
}


// Prototypes:
// #####################Last change: 04/04/98 17h48mn#########################



#define EBITSTRM_HPP_INCLUDED
#endif


// ########################### EOF EBITSTRM.HPP ##############################
// ########################### EOF EBITSTRM.HPP ##############################
// ########################### EOF EBITSTRM.HPP ##############################

