// ###########################################################################
// File: EDATASTR.CPP
// (c)Copyright 08/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// ###########################################################################

/* HST:
 * 04/04/98: - Now derivated from EStream.
 *           - added getByte() & putByte().
 * 09/08/98: - BugFix: Added '\0' to string loaded using operator >>(char *&v).
 * 03/10/98: - Add readCStr( char *&s )
 *           - Add readCStr( EString &s )
 *           - Add writeCStr( const char *s )
 */

#include <string.h>
#include <stdlib.h>
#include "usuel.hpp"
#include "edatastr.hpp"
#ifndef EDATASTREAM_NOSUPPORT_ESTRING
#  include "estring.hpp"
#endif


/** Constructs a data stream with a device.
  * Constructs a data stream for device \a device.
  * @param pDevice device used by the data stream.
  */
EDataStream::EDataStream( EIODevice *pDevice ) :
    EStream( pDevice )
{
}

/** Read a block.
  * @param pData data to read.
  * @param nLen length of data to read in byte.
  */
void EDataStream::readBlock( void *pData, SL nLen )
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    UB *p = (UB *) pData;
    while ( nLen-- )
      *this << *p++;
  }
#endif
  device()->readBlock( pData, nLen );
}

/** Write a block.
  * @param pData data to write.
  * @param nLen length of data to write in byte.
  */
void EDataStream::writeBlock( const void *pData, SL nLen )
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    UB *p = (UB *) pData;
    while ( nLen-- )
      *this >> *p++;
  }
#endif
  device()->writeBlock( pData, nLen );
}


EDataStream& EDataStream::operator >>(SW& v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    v = readInteger();
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[2];
    UB *p = (UB *)&v;
    readBlock( tmp, 2 );
    *p++ = tmp[1];
    *p = tmp[0];
  }
  else
    readBlock( &v, 2 );
  return *this;
}

EDataStream& EDataStream::operator >>(SL& v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    v = readInteger();
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[4];
    UB *p = (UB *)&v;
    readBlock( tmp, 4 );
    *p++ = tmp[3];
    *p++ = tmp[2];
    *p++ = tmp[1];
    *p = tmp[0];
  }
  else
    readBlock( &v, 4 );
  return *this;
}

EDataStream& EDataStream::operator >>(float& v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    v = (float)readDouble();
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[4];
    UB *p = (UB *)&v;
    readBlock( tmp, 4 );
    *p++ = tmp[3];
    *p++ = tmp[2];
    *p++ = tmp[1];
    *p = tmp[0];
  }
  else
    readBlock( &v, 4 );
  return *this;
}

EDataStream& EDataStream::operator >>(double& v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    v = readDouble();
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[8];
    UB *p = (UB *)&v;
    readBlock( tmp, 8 );
    *p++ = tmp[7];
    *p++ = tmp[6];
    *p++ = tmp[5];
    *p++ = tmp[4];
    *p++ = tmp[3];
    *p++ = tmp[2];
    *p++ = tmp[1];
    *p = tmp[0];
  }
  else
    readBlock( &v, 8 );
  return *this;

}

EDataStream& EDataStream::operator >>(char *& v)
{
  SL len;
  *this >> len;
  v = new char[len +1];
  readBlock( v, len );
  v[ len ] = 0;
  return *this;
}


EDataStream& EDataStream::operator <<(SW v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    writeInteger( v );
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[2];
    UB *p = (UB *)&v;
    tmp[1] = *p++;
    tmp[0] = *p;
    writeBlock( tmp, 2 );
  }
  else
    writeBlock( &v, 2 );
  return *this;
}

EDataStream& EDataStream::operator <<(SL v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    writeInteger( v );
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[4];
    UB *p = (UB *)&v;
    tmp[3] = *p++;
    tmp[2] = *p++;
    tmp[1] = *p++;
    tmp[0] = *p;
    writeBlock( tmp, 4 );
  }
  else
    writeBlock( &v, 4 );
  return *this;
}

EDataStream& EDataStream::operator <<(float v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    writeDouble( v );
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[4];
    UB *p = (UB *)&v;
    tmp[3] = *p++;
    tmp[2] = *p++;
    tmp[1] = *p++;
    tmp[0] = *p;
    writeBlock( tmp, 4 );
  }
  else
    writeBlock( &v, 4 );
  return *this;
}

EDataStream& EDataStream::operator <<(double v)
{
#ifdef PRINTABLE_STREAM
  if ( isPrintable() )
  {
    writeDouble( v );
    return *this;
  }
#endif
  if ( needSwap() )
  {
    UB tmp[8];
    UB *p = (UB *)&v;
    tmp[7] = *p++;
    tmp[6] = *p++;
    tmp[5] = *p++;
    tmp[4] = *p++;
    tmp[3] = *p++;
    tmp[2] = *p++;
    tmp[1] = *p++;
    tmp[0] = *p;
    writeBlock( tmp, 8 );
  }
  else
    writeBlock( &v, 8 );
  return *this;
}

EDataStream& EDataStream::operator <<(const char * v)
{
  SL len = strlen( v );
  *this << (SL)len;
  writeBlock( v, len );
  return *this;
}


EDataStream &
EDataStream::readCStr( char * &s)
{
  SL nMaxLen = 16;
  SL nLen = 0;
  s = (char *)emalloc( sizeof(char) * (nMaxLen+1) );
  char c;
  while ( c = getByte() )
  {
    if ( nLen == nMaxLen )
    {
      nMaxLen += 16;
      s = (char *)erealloc( s, sizeof( char ) * (nMaxLen +1) );
    }
    if ( s )
      s[ nLen ] = c;
    nLen++;
  }

  s = (char *)erealloc( s, sizeof( char ) * (nLen +1) );
  if ( s )
    s[ nLen ] = 0;

  return *this;
}


EDataStream &
EDataStream::writeCStr( const char *s )
{
  if ( !s )
    putByte( 0 );
  else
  {
    writeBlock( s, strlen( s ) +1 );
  }

  return *this;
}


#ifndef EDATASTREAM_NOSUPPORT_ESTRING
EDataStream &
EDataStream::readCStr( EString &s)
{
  s.setNull();
  UB c;
  while ( c = getByte() )
  {
    s += c;
  }

  return *this;
}
#endif
