// ###########################################################################
// File: ELIST.CPP
// (c)Copyright 04/1997, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// Version 1.2
// ###########################################################################

/* HST:
 * 09/11/27: - remove bug in nodeAt(). When walking from the end, start from
 *             nb instead of nb-1. (twice in function).
 *           - check run for at method -> works fine.
 * 29/03/98: - now compile with VC++5.0. -> When using VC, friend is
 *             remove and all attribute are public (Arrgghhgg!).
 * 02/04/98: - count() is now const.
 *           - now compile with new boolean def. (true instead of TRUE).
 * 09/08/98: - Added alias for iterator in EList.
 * 16/09/98: - Change EABS -> EDIST.
 * 14/09/99: - Removes delete element. (delete type void * !!!)
 */

#include "usuel.hpp"
#include "elist.hpp"

/*  Testing routing...
void checkElist()
{
  printf("Build EList\n");
  EList<L> l;
  L cpt, no=0;

  for (cpt =0; cpt <1000; ++cpt)
  {
    L *v = new L(cpt);
    l.insertAtEnd( v );
  }
  --cpt;

  printf("Testing\n");
  while (! kbhit() )
  {
    printf("Test %d, Previous: cpt=%d, ", no, cpt );
    cpt = rand() % 1000;
    printf(" New cpt=%d\n", cpt );
    if ( *(l.at( cpt )) != cpt )
    {
      printf("Test failed!\n");
      return;
    }
    no++;
  }

}
*/

void *EGenList::at(UL pos)
{
  ENode *n = nodeAt(pos);
  if (n)
  {
    cur = n;
    idxCur = pos;
    return n->elm;
  }
  return ENIL;
}

void EGenList::init()
{
  head = ENIL;
  tail = ENIL;
  cur = ENIL;
  idxCur = 0;
  nb = 0;
  mine = true;
}

EGenList::EGenList()
{
  init();
}

EGenList::~EGenList()
{
  reset();
}

void EGenList::reset()
{
  while (!isEmpty())
  {
    if (mine)
      deleteFirst();
    else
      removeFirst();
  }
}

EGenList::ENode *EGenList::nodeAt(UL pos)
{
  ENode *n;
  UL cpt;

  if (pos == 0)
    return head;
  if (pos == nb)
    return tail;
  if (pos > nb)
    return ENIL;

  if (cur)    // current ptr exist !
  {
    UL distToCur = EDIST(pos, idxCur);
    UL distToBeg = pos;
    UL distToEnd = nb -pos;

    if (distToBeg < distToCur)    // => distToBeg < distToEnd
    {                             // look for from beginning
      for (cpt = 0, n = head; cpt < pos; cpt++, n = n->next);
    }
    else
    {
      if (distToEnd < distToCur)    // look for from the end
      {
        for (cpt = nb-1, n = tail; cpt > pos; cpt--, n = n->prev);
      }
      else                        // distToCur is the nearest
      {
        if (idxCur > pos)         // look for backward from current
          for (cpt = idxCur, n = cur; cpt > pos; cpt--, n = n->prev);
        else                      // look for forward from current
          for (cpt = idxCur, n = cur; cpt < pos; cpt++, n = n->next);
      }
    }
    return n;
  }
  else      // current do not exist !
  {
    if (pos < nb-pos)   // nearer of the beginning
    {
      for (cpt = 0, n = head; cpt < pos; cpt++, n = n->next);
    }
    else                // nearer of the end
    {
      for (cpt = nb-1, n = tail; cpt > pos; cpt--, n = n->prev);
    }
    return n;
  }
}

void EGenList::insertAt(void *elm, UL pos)
{
  ENode *n = new ENode;
  n->prev = ENIL;
  n->next = ENIL;
  n->elm = elm;

  if (pos == 0)       // insert at beginning
  {
    n->next = head;
    if (head)
      head->prev = n;
    head = n;
    if (!tail)
      tail = head;
  }
  else
  {
    if (pos < nb)   // insert in middle
    {
      ENode *prevNode;
      prevNode = nodeAt(pos);
      prevNode->prev = n;             // we are sure that we are nor at the end nor at the beginning.
      n->next = prevNode;
    }
    else            // insert at end
    {
      n->prev = tail;
      if (tail)
        tail->next = n;
      tail = n;
      if (!head)
        head = tail;
    }
  }

  nb++;

  if (cur)  // if current exist
  {         // update position
    if (idxCur >= pos)
      idxCur++;
  }
  else      // else set current to head
  {
    cur = head;
    idxCur = 0;
  }
}

void *EGenList::removeLast()
{
  if (nb)
    return removeAt(nb-1);
  return ENIL;
}

void *EGenList::removeAt(UL pos)
{
  ENode *n = ENIL;
  ENode *nextCur = cur;
  UL nextIdxCur = idxCur;
  void *elm;

  if (pos >= nb  ||  !nb)      // out of bound
    return ENIL;

  if (cur)
  {
    if (idxCur == pos)     // Remove the current element
    {
      if (cur->next)      // seek to next if possible
        nextCur = cur->next;
      else
      {
        if (cur->prev)    // otherwise seek to previous if possible
        {
          nextCur = cur->prev;
          nextIdxCur = idxCur -1;
        }
        else              // otherwise disable current
        {
          nextCur = ENIL;
          nextIdxCur = 0;
        }
      }
    }
    else
    {
      if (pos < idxCur)   // else if remove before current the update
        nextIdxCur = idxCur -1;         // index
    }
  }

  if (pos == 0)     // remove at beginning
  {
    n = head;
    head = head->next;
    if (nb == 1)        // also need to change tail
      tail = head;      // (ENIL)
  }
  else
  {
    if (pos == nb -1)   // remove at the end
    {
      n = tail;
      tail = tail->prev;
    }
    else                // remove in middle
    {
      n = nodeAt(pos);    // this have a predecessor & a successor !
      assert(n);        // oops !
      n->prev->next = n->next;    // unchained node n !
      n->next->prev = n->prev;
    }
  }

  elm = n->elm;     // get element
  delete n;         // and delete node

  cur = nextCur;        // update current
  idxCur = nextIdxCur;
  nb--;

  return elm;
}

void EGenList::deleteFirst()
{
  void *elm = removeFirst();
//  if (elm)
//    delete elm;
}

void EGenList::deleteLast()
{
  void *elm = removeLast();
//  if (elm)
//    delete elm;
}

void EGenList::deleteAt(UL pos)
{
  void *elm = removeAt(pos);
//  if (elm)
//    delete elm;
}



// ############################ EOF ELIST.CPP ################################
// ############################ EOF ELIST.CPP ################################
// ############################ EOF ELIST.CPP ################################

