// ###########################################################################
// File: ESTRING.CPP
// (c)Copyright 04/1998, Lepilleur Baptiste (Aka Elyon from Zuul Design)
// Version 0.31
// ###########################################################################
/* HST:
 * - 17/09/00: - Add append( const EString &str, SL nCount )
 * - 29/05/00: - Modified contains() [behavior when searching 0].
 * - 29/03/98: - Add contains().
 * - 04/04/98: - Remove derivation from EDynArray<Char>
 *             - Near full rewrite. Memory usage is lower (8 bytes per string)!
 *             - resize() is now public.
 *             - indirect access to string buffer through string(). Should make
 *               change for sharing easier.
 *             - add operator =
 *             - add operator += for const char * and EString&.
 *             - add append()
 *             - add left()
 *             - add mid()
 *             - add right()
 *             - add insert()
 *             - add remove()
 *             - add truncate()
 *             - add pos()
 *             - add posLast()
 *             - add operator +()
 *             - add comparaison operator ==,!=, <, <=, >, >=
 *             - add cmp()
 *             - add icmp()
 *             - add cast const char*()
 *             - add lower()
 *             - add upper()
 *             - add toLower()
 *             - add toUpper()
 * 06/04/98:   - add cmpn()
 * 26/04/98:   - fix a signed/unsigned warning under VC in copy()
 * 26/07/98:   - add makeHashKey()
 *             - add imakeHashKey()
 *             - add dquoted()   [to use for SQL query]
 *             - add quoted()
 * 08/10/1998: - BugFix: assert( sz <..) misplaced .
 * 19/10/1998: - BugFix: forgot destructor ~EString() !!!
 * 26/10/1998: - add function operator << & >> for storage in datastream.
 * 28/11/1998: - add the "const attribute" to mid(), left(), right()
 *             - BugFix: cmpn() when nNb = 0 and this string was empty, returned
 *                       -1 instead of 0 !
 * 12/12/1998: - add lastChar()
 *             - BugFix: imakeHashKey() & makeHashKey(), case where string is NULL.
 * 13/12/1998: - add isInteger()
 *             - add isSignedInteger()
 *             - add valSignedInt()
 *             - add valInt()
 *             - add isHexaInt()
 *             - add valHexaInt()
 *             - add trimLeft()
 *             - add trimRight
 *             - add trimSpaces
 * 15/12/1998: - add setHexa()
 * 18/12/1998: - BugFix: toLower() & toUpper(): forgot case string=NULL! 
 * 24/02/1999: - add operators: EString operator +( const EString, ...)
 *             - change posLast() to posLast( char c, SL nPosLast ), for reverse search.
 * 26/02/1999: - add isUnicode()
 *             - add isAnsi()
 *             - add byteLength()
 *             - add ensureNotNull()
 * 03/03/1999: - BugFix: makeHashKey() was using at() while iterating on the terminal 0.
 * 11/09/1999: - add static nullString member.
 * 12/09/1999: - add SL pos( SL nPos, const EString &strToFind ) const
 *             - add SL pos( const EString &strToFind ) const
 */

/* To do:
 * replace atoi() by a hand-made function.
 * - Ajouter les operateurs standards, filter
 * - Add shared string -> allow to return string in a fast way.
 */

/* Test:
 * [OK] EString( const char *text )
 * [OK] EString( const EString& str )
 * [  ] EString( char c )
 * [OK] EString()
 * [OK] ~EString()
 * [  ] EString& copy( const char *s, SL nLen )
 * [OK] EString& copy( const char *s )
 * [OK] EString& copy( const EString& str )
 * [OK] bool isNull() const
 * [OK] void setNull()
 * [OK] SL length() const
 * [OK] const char *datas() const
 * [OK] bool contains( char c ) const
 * [OK] EString& operator +=( char c )
 * [OK] EString& operator +=( const char *s )
 * [  ] EString& operator +=( const EString &str )
 * [OK] char &at( SL nIndex )
 * [OK] char &operator[]( SL nIndex )
 * [OK] char at( SL nIndex ) const
 * [OK] char operator[]( SL nIndex ) const
 * [OK] void resize( SL nSize )
 * [OK] void init()
 * [  ] EString& operator =( char c )
 * [OK] EString& operator =( const char *s )
 * [OK] EString& operator =( const EString& str )
 * [OK] EString& append( char c )
 * [  ] EString& append( const char *s )
 * [  ] EString& append( const EString &str )
 * [OK] EString left( SL nNb )
 * [OK] EString right( SL nNb )
 * [OK] EString mid( SL nPos, SL nNb )
 * [  ] EString &insert( SL nPos, char c )
 * [  ] EString &insert( SL nPos, const char *s )
 * [  ] EString &insert( SL nPos, const EString& str )
 * [OK] EString &remove( SL nPos, SL nNb =1)
 * [OK] EString &truncate( SL nPos )
 * [OK] SL pos( char c ) const
 * [OK] SL pos( SL nPos, char c ) const
 * [  ] SL pos( const EString &strToFind ) const
 * [  ] SL pos( SL nPos, const EString &strToFind ) const
 * [OK] SL posLast( char c, SL nPosStart ) const
 * [  ] EString operator +( const EString &str2 )
 * [OK] EString operator +( const char *s2 )
 * [  ] EString operator +( char c )
 * [OK] SL cmp( const EString& str ) const
 * [OK] SL icmp( const EString& str ) const
 * [OK] bool operator ==( const EString &str ) const
 * [  ] bool operator !=( const EString &str ) const
 * [  ] bool operator <( const EString &str ) const
 * [  ] bool operator <=( const EString &str ) const
 * [  ] bool operator >( const EString &str ) const
 * [  ] bool operator >=( const EString &str ) const
 * [  ] SL cmp( const char *s ) const
 * [OK] SL icmp( const char *s ) const
 * [  ] bool operator ==( const char *s ) const
 * [  ] bool operator !=( const char *s ) const
 * [  ] bool operator <( const char *s ) const
 * [  ] bool operator <=( const char *s ) const
 * [  ] bool operator >( const char *s ) const
 * [  ] bool operator >=( const char *s ) const
 * [  ] SL cmp( char c ) const
 * [  ] SL icmp( char c ) const
 * [  ] bool operator ==( char c ) const
 * [  ] bool operator !=( char c ) const
 * [  ] bool operator <( char c ) const
 * [  ] bool operator <=( char c ) const
 * [  ] bool operator >( char c ) const
 * [  ] bool operator >=( char c ) const
 * [OK] operator const char *() const
 * [OK] EString lower() const
 * [  ] EString upper() const
 * [OK] EString &toLower()
 * [  ] EString &toUpper()
 * [  ] EString &toLower()  { WATCOMC }
 * [  ] EString &toUpper()  { WATCOMC }
 * [OK] SL cmpn( const EString &str, SL nNb ) const
 * [  ] SL cmpn( const char *s, SL nNb ) const
 * [OK] UL makeHashKey() const
 * [OK] UL imakeHashKey() const
 * [  ] EString dquoted() const
 * [  ] EString quoted() const
 * [OK] EDataStream &operator <<( EDataStream &s, const EString &str )
 * [OK] EDataStream &operator >>( EDataStream &s, EString &str )
 * [OK] char lastChar() const
 * [  ] bool isInt() const
 * [OK] bool isSignedInt() const
 * [OK] SL valSignedInt() const
 * [  ] UL valInt() const
 * [OK] bool isHexaInt() const
 * [OK] UL valHexaInt() const
 * [OK] EString &trimLeft()
 * [OK] EString &trimRight()
 * [OK] EString &trimSpaces()
 * [  ] void setHexa( UL nNum, const char *sPrefix, SL nNumDigit, SL nLowerDigitNo )
 * [OK] EString operator +( const EString &str1, const EString &str2 )
 * [OK] EString operator +( const EString &str1, const TCHAR *s2 )
 * [OK] EString operator +( const EString &str1, TCHAR c )
 */

#include <ctype.h>
#include "usuel.hpp"
#include "estring.hpp"
#include "EDataStr.hpp"

/// must be a power of 2 !!!
const SL EString::_nGranularity = 16;
const EString EString::nullString;

/// initialize members.
void EString::init()
{
  _nLength =0;
  _sString = ENIL;
}

/** Resize the string.
  * @param nSize new size of the string.
  */
void EString::resize( SL nSize )
{
  EASSERT( nSize >= 0 );
  static SL sz = 0;

  if ( !nSize )
  {
    efree( _sString );
    _nLength = 0;
    _sString = ENIL;
  }
  else
  {
    SL nNewLen = nSize & ~(_nGranularity-1);
    if ((_nLength & ~(_nGranularity-1)) != nNewLen  ||  !_sString )
    {
      EASSERT( nNewLen+_nGranularity > nSize );
      sz = nNewLen+_nGranularity;
      _sString = (char *)erealloc( _sString, nNewLen + _nGranularity );
      EASSERT( sz >= nSize+1 );
    }
    _nLength = nSize;
    _sString[ nSize ] = 0;
  }
}


void 
EString::ensureNotNull()
{
  if ( _sString  &&  _nLength )
    return;

  _sString = (TCHAR *)erealloc( _sString, sizeof(TCHAR) );
  _sString[ 0 ] =0;
}


/** Copy a string.
  * Copy the zero terminated string \a s.
  * @param s string to copy
  * @return the string.
  */
EString& EString::copy( const char *s )
{
  if ( s )
  {
    SL nLength = strlen( s );
    resize( nLength );
    memcpy( string(), s, length() );
  }
  else
    setNull();
  return *this;
}

/** Copy a string.
  * Copy the string \a s.
  * @param s string to copy
  * @return the string.
  */
EString& EString::copy( const EString& str )
{
  resize( str.length() );
  if ( !isNull() )
    memcpy( string(), str.datas(), length() );
  return *this;
}

/** Copy a string.
  * Copy the string \a s of length \a len characters.
  * @param s string to copy
  * @param nLen length of the string (without the terminal 0).
  * @return the string.
  */
EString& EString::copy( const char *s,
                        SL nLen )
{
  if ( s )
  {
    assert( nLen <= (SL)strlen( s ) );
    resize( nLen );
    memcpy( string(), s, length() );
  }
  else
    setNull();
  return *this;
}

/** Append a string to the string.
  * @param s string to append.
  * @return reference to this string.
  */
EString& EString::operator +=( const char *s )
{
  if ( !s )
    return *this;

  SL nLen = strlen( s );
  SL nPos = length();
  resize( length() +nLen );
  memcpy( &string()[ nPos ], s, nLen );
  return *this;
}

/** Append a string to the string.
  * @param str string to append.
  * @return reference to this string.
  */
EString& EString::operator +=( const EString &str )
{
  if ( str.isNull() )
    return *this;

  SL nPos = length();
  SL nLen = str.length();     // in case str == this !
  resize( length() +nLen );
  memcpy( &string()[ nPos ], str.datas(), nLen );
  return *this;
}


EString &
EString::append( const EString &str,
                 SL nCount )
{
  if ( nCount > 0  &&  !str.isNull() )
  {
    SL nPos = length();
    SL nLen = str.length();     // in case str == this !
    resize( length() + nLen * nCount );

    while ( --nCount >= 0 )
    {
      memcpy( &string()[ nPos ], str.datas(), nLen );
      nPos += nLen;
    }
  }

  return *this;
}


/** Return some of the first char of the string.
  * Copy the \a nNb first char of the string.
  * @param nNb Number of char to copy ( nNb >= 0 ).
  * @return A string with the nNb left char of the string.
  */
EString 
EString::left( SL nNb ) const
{
  assert( nNb >= 0 );
  EString strResult;
  strResult.copy( string(), EMIN( nNb, length() ) );
  return strResult;
}

/** Return some of the last char of the string.
  * Copy the \a nNb last char of the string.
  * @param nNb Number of char to copy ( nNb >= 0 ).
  * @return A string with the \a nNb right char of the string.
  */
EString EString::right( SL nNb ) const
{
  assert( nNb >= 0 );
  EString strResult;
  SL nPos = EMAX( length() -nNb, 0);
  strResult.copy( string() +nPos , EMAX( length()-nPos, 0 ) );
  return strResult;
}

/** Return some char from the middle of the string.
  * Copy \a nNb char from index \a nIndex.
  * @param nPos Index of the first char to copy. ( nPos >= 0 ).
  * @param nNb Number of char to copy. If nNb < 0 then char are copied
  *            from nPos to length()-nNb.
  * @return A string with the some char of the string.
  */
EString EString::mid( SL nPos, SL nNb ) const
{
  assert( nPos >= 0 );
  EString strResult;

  if ( nNb < 0 )
  {
    nNb = length() +nNb +1 -nPos;
  }
  if ( nNb > 0 )
  {
    if ( nPos < length() )
      strResult.copy( string() +nPos, EMIN( nNb, length() -nPos ) );
  }
  return strResult;
}

/** Insert a char in the string.
  * @param nPos position where the char is inserted. (length()>=nIndex >=0)
  * @param c char to insert.
  */
EString &EString::insert( SL nPos, char c )
{
  assert( nPos >=0 );
  assert( nPos <= length() );

  resize( length() +1 );
  memmove( &at( nPos+1 ), &at( nPos), length() -nPos-1 );
  at( nPos ) = c;

  return *this;
}

/** Insert a string in the string.
  * @param nPos position where the string is inserted. (length()>nIndex >=0)
  * @param s string to insert. if \a s is null then nothing is done.
  */
EString &EString::insert( SL nPos, const char *s )
{
  assert( nPos >=0 );
  assert( nPos <= length() );

  if ( !s )
    return *this;

  SL nLen = strlen( s );
  if ( !nLen )
    return *this;

  resize( length() +nLen );
  memmove( &at( nPos+nLen ), &at( nPos ), length() -nPos-nLen );
  memcpy( &at( nPos ), s, nLen );

  return *this;
}

/** Insert a string in the string.
  * @param nPos position where the string is inserted. (length()>nIndex >=0)
  * @param s string to insert.
  */
EString &EString::insert( SL nPos, const EString& str )
{
  assert( nPos >=0 );
  assert( nPos <= length() );
  assert( &str != this );

  SL nLen = str.length();
  memmove( &at( nPos+nLen ), &at( nPos ), length() -nPos-nLen );
  memcpy( &at( nPos ), str.string(), nLen );

  return *this;
}

/** Remove some char in the string.
  * @param nPos position of the character to delete (0<=nPos).
  *              If nPos > length() then nothing is done.
  * @param nNb number of chars to delete. if nNb < 0 then all char are
  *            remove to position length()+nNb. A value of -1 for nNb will
  *            remove all char until the end of the string.
  * @return reference on this string.
  */
EString &EString::remove( SL nPos, SL nNb )
{
  assert( nPos >= 0 );

  if ( nNb < 0 )
    nNb = length() +nNb +1 -nPos;

  if ( nNb <= 0  ||  nPos > length() )
    return *this;

  // test for algo bug!
  assert( nPos >=0  &&  nPos <= length()  &&  nPos+nNb <= length() );

  SL nLeft = length() -nNb -nPos;
  assert( nLeft >= 0 );
  if ( nLeft )
    memcpy( &at( nPos ), &at( nPos+nNb ), nLeft );
  resize( length() -nNb );
  return *this;
}


SL 
EString::count( char c ) const
{
  SL nPos =0;
  SL nOcc = -1;
  do
  {
    ++nOcc;
    nPos = pos( nPos, c ) +1;
  }
  while ( nPos > 0 );
  return nOcc;
}


/** Search the first occurrence of a char in the string.
  * @param c char to search for.
  * @return index of the occurrence in the string. -1 if none found.
  */
SL EString::pos( char c ) const
{
  if ( isNull() )
    return -1;

  char *pOccurrence = strchr( string(), c );
  return pOccurrence ? pOccurrence-string() : -1;
}

/** Search the first occurrence of a char starting at a given position.
  * @param nPos position the search begin at. nPos >= 0.
  * @param c char to search for.
  * @return index of the occurrence in the string. -1 if none found.
  */
SL EString::pos( SL nPos, char c ) const
{
  if ( isNull()  ||  nPos >= length() )
    return -1;

  char *pOccurrence = strchr( &string()[nPos], c );
  return pOccurrence ? pOccurrence-string() : -1;
}


SL 
EString::pos( SL nPos, const EString &strToFind ) const
{
  if ( isNull()  ||  nPos >= length() )
    return -1;

  char *pOccurence = strstr( &string()[nPos], strToFind.datas() );
  return pOccurence ? pOccurence -string() : -1;
}


/** Search the last occurrence of a char in the string.
  * @param c char to search for.
  * @return index of the occurrence in the string. -1 if none found.
  */
SL EString::posLast( char c, 
                     SL nPosStart ) const
{
  if ( isNull()  ||  nPosStart == 0 )
    return -1;

  if ( nPosStart == -1  ||  nPosStart >= length() )
  {
    char *pOccurrence = strrchr( string(), c );
    return pOccurrence ? pOccurrence-string() : -1;
  }
  else
  {
    if ( nPosStart < 0 )
      nPosStart += length();

    const TCHAR *sFirst = datas();
    const TCHAR *sPos = datas() + nPosStart;

    while ( *sPos != c  &&      
            sPos != sFirst )
      sPos--;

    if ( *sPos == c )   // found !
      return sPos - sFirst;
    
    return -1;          // not found !
  }
}

/** Concat two string.
  * @param str1 first string.
  * @param str2 second string.
  * @return a string containing the concatenation of \a str1  and \a str2.
  */
EString EString::operator +( const EString &str2 )
{
  EString strResult( *this );
  strResult += str2;
  return strResult;
}

/** Concat two string.
  * @param str1 first string.
  * @param s2 second string. If s2 is NULL then the result string is str1.
  * @return a string containing the concatenation of \a str1  and \a s2.
  */
EString EString::operator +( const char *s2 )
{
  EString strResult( *this );
  strResult += s2;
  return strResult;
}

/** Concat a string and a char.
  * @param str1 first string.
  * @param c char to concat to str1.
  * @return a string containing the concatenation of \a str1  and \a c.
  */
EString EString::operator +( char c )
{
  EString strResult( *this );
  strResult += c;
  return strResult;
}


/** Compare this string to another string.
  * This is a case sensitive compare.
  * @param str string this string is compared to.
  * @return 0 is the 2 string are equal, or both null.
  *         >0 if this string is greater than \a str.
  *         <0 if this string if less than \a str.
  */
SL EString::cmp( const EString& str ) const
{
  if ( isNull() )
  {
    if ( str.isNull() )
      return 0;
    return -1;
  }
  if ( str.isNull() )
    return 1;
  return strcmp( string(), str.string() );
}


/** Compare this string to another string.
  * This is a case sensitive compare.
  * @param s string this string is compared to.
  * @return 0 is the 2 string are equal, or both null.
  *         >0 if this string is greater than \a s.
  *         <0 if this string if less than \a s.
  */
SL EString::cmp( const char *s ) const
{
  if ( isNull() )
  {
    if ( !s  ||  !s[0] )
      return 0;
    return -1;
  }
  if ( !s  ||  !s[0] )
    return 1;
  return strcmp( string(), s );
}

/** Compare this string to a char.
  * This is a case sensitive compare.
  * @param c char the string is compared to.
  * @return 0 is the string as a length of 1 and the first char is equal to
  *           \a c.
  *         >0 if this string is greater than c.
  *         <0 if this string if less than c.
  */
SL EString::cmp( char c ) const
{
  if ( isNull() )
    return -1;
  SL nDelta = (SL)(at(0) -c);
  if ( nDelta == 0 )
    return (length() >1 ) ? 1 : 0;
  return nDelta;
}

/** Compare this string to another string.
  * This is a case insensitive compare.
  * @param str string this string is compared to.
  * @return 0 is the 2 string are equal, or both null.
  *         >0 if this string is greater than \a str.
  *         <0 if this string if less than \a str.
  */
SL EString::icmp( const EString& str ) const
{
  if ( isNull() )
  {
    if ( str.isNull() )
      return 0;
    return -1;
  }
  if ( str.isNull() )
    return 1;
  return stricmp( string(), str.string() );
}

/** Compare this string to another string.
  * This is a case insensitive compare.
  * @param s string this string is compared to.
  * @return 0 is the 2 string are equal, or both null.
  *         >0 if this string is greater than \a s.
  *         <0 if this string if less than \a s.
  */
SL EString::icmp( const char *s ) const
{
  if ( isNull() )
  {
    if ( !s  ||  !s[0] )
      return 0;
    return -1;
  }
  if ( !s  ||  !s[0] )
    return 1;
  return stricmp( string(), s );
}

/** Compare this string to a char.
  * This is a case insensitive compare.
  * @param c char the string is compared to.
  * @return 0 is the string as a length of 1 and the first char is equal to
  *           \a c.
  *         >0 if this string is greater than c.
  *         <0 if this string if less than c.
  */
SL EString::icmp( char c ) const
{
  if ( isNull() )
    return -1;
  SL nDelta = (SL)(tolower(at(0)) -tolower(c));
  if ( nDelta == 0 )
    return (length() >1 ) ? 1 : 0;
  return nDelta;
}

/** Compare a given number of chars in the two string.
  * @param str string this string is compared to.
  * @param nNb number of chars to compare (nNb >= 0).
  * @return 0 is the \a nNb first chars of both string are equal.
  *         >0 if this string is greater than \a str.
  *         <0 if this string if less than \a str.
  */
SL EString::cmpn( const EString &str, SL nNb ) const
{
  assert( nNb >= 0 );

  if ( nNb == 0 )
    return 0;

  if ( isNull() )
  {
    if ( str.isNull() )
      return 0;
    return -1;
  }
  if ( str.isNull() )
    return 1;

  return strncmp( string(), str.string(), nNb );
}

/** Compare a given number of chars in the two string.
  * @param s string this string is compared to.
  * @param nNb number of chars to compare (nNb >= 0).
  * @return 0 is the \a nNb first chars of both string are equal.
  *         >0 if this string is greater than \a s.
  *         <0 if this string if less than \a s.
  */
SL EString::cmpn( const char *s, SL nNb ) const
{
  assert( nNb >= 0 );

  if ( nNb == 0 )
    return 0;

  if ( isNull() )
  {
    if ( !s  ||  !s[0] )
      return 0;
    return -1;
  }
  if ( !s  ||  !s[0] )
    return 1;

  return strncmp( string(), s, nNb );
}

/** Get a copy of the string where all char are in lower case.
  * @return a copy of the string in lower case.
  */
EString EString::lower() const
{
  EString s( *this );
  s.toLower();
  return s;
}

/** Get a copy of the string where all char are in upper case.
  * @return a copy of the string in upper case.
  */
EString EString::upper() const
{
  EString s( *this );
  s.toUpper();
  return s;
}

#ifndef __WATCOMC__
/** Convert all char of the string to lower case.
  * @return reference on this string.
  */
EString &EString::toLower()
{
  if ( length() )
  {
    char *s = string();
    while ( *s++ = tolower( *s ) );
  }
  return *this;
}

/** Convert all char of the string to upper case.
  * @return reference on this string.
  */
EString &EString::toUpper()
{
  if ( length() )
  {
    char *s = string();
    while ( *s++ = toupper( *s ) );
  }
  return *this;
}
#endif



/** Make a case sensitive key for hashtable.
 * The key is a numeric value build from the string content.
 * @return key value.
 */
UL 
EString::makeHashKey() const
{
  UL k = 0;
  const char *szString = string();
  char c;

  if ( szString )
  {
    while ( c = *szString++ )
    {
      k =  ( k << 5 ) + ( k >> (32-5) ) + c;
    }
  }

  return k;
}

/** Make a case insensitive key for hashtable.
 * The key is a numeric value build from the string content. The key
 * returned is computed by applying tolower on each char. Therefore
 * the key value is the same for a upper case and a lower case string.
 * @return key value.
 */
UL 
EString::imakeHashKey() const
{
  UL k = 0;
  const char *szString = string();
  char c;

  if ( szString )
  {
    while ( c = tolower( *szString++ ) )
    {
      k =  ( k << 5 ) + ( k >> (32-5) ) + c;
    }
  }

  return k;
}

EString 
EString::dquoted() const
{
	EString str( '\"' );
	const char *szText = string();
  char c;
	while ( c = *szText )
	{
		if ( c == '"' )
			str += "\"\"\"";
		else
			str += c;
	}
	str += '"';
	return str;
}

EString 
EString::quoted() const
{
	EString str( '\'' );
	const char *szText = string();
  char c;
	while ( c = *szText )
	{
		if ( c == '\'' )
			str += "''" ;
		else
			str += c;
	}
	str += '\'';
	return str;
}


bool 
EString::isInt() const
{
  if ( !length() )
    return false;

  SL nIdx;
  for ( nIdx =0; nIdx < length(); ++nIdx )
  {
    if ( at( nIdx ) < '0'  ||  at( nIdx ) > '9' )   // not a digit
      return false;
  }

  return true;
}


bool 
EString::isSignedInt() const
{
  if ( !length() )
    return false;

  SL nIdx =0;
  if ( at( nIdx ) == '-'  ||  at( nIdx ) == '+' )   // skip sign
  {
    if ( length() == 1 )                // must not be only a sign
      return false;

    nIdx++;
  }

  for ( ; nIdx < length(); ++nIdx )
  {
    if ( at( nIdx ) < '0'  ||  at( nIdx ) > '9' )   // not a digit
      return false;
  }

  return true;
}


SL 
EString::valSignedInt() const
{
  return atoi( string() );
}


UL 
EString::valInt() const
{
  return (UL)atoi( string() );
}


bool 
EString::isHexaInt() const
{
  if ( isNull() )
    return false;

  SL nIdx;
  for ( nIdx =0; nIdx < length(); ++nIdx )
  {
    char c = tolower( at( nIdx ) );
    if (  ( c < '0'  ||  c > '9' )  &&
          ( c < 'a'  ||  c > 'f' ) )
      return false;
  }

  return true;
}


UL 
EString::valHexaInt() const
{
  UL nVal = 0;

  if ( isNull() )
    return 0;

  SL nIdx;
  for ( nIdx =0; nIdx < length(); ++nIdx )
  {
    char c = tolower( at( nIdx ) );

    if ( c >= '0'  &&  c <= '9' )
      nVal = ( nVal * 16 ) + ( c - '0');
    else if ( c >= 'a'  &&  c <= 'f' )
      nVal = ( nVal *16 ) + ( c -'a' + 10 );
    else
      return 0;
  }

  return nVal;
}


EString &
EString::trimLeft()
{
  SL nLength = length();
  SL nIdx =0;
  while ( nIdx < nLength  &&  
          at( nIdx ) < 33 )
    nIdx++;
  
  if ( nIdx > 0  &&  nIdx < nLength )
  {
    memcpy( string(), &at( nIdx ), nLength -nIdx );
    resize( nLength -nIdx );
  }

  return *this;
}


EString &
EString::trimRight()
{
  SL nIdx = length();
  while ( nIdx > 0  &&  
          at( nIdx -1 ) < 33 )
    nIdx--;
  
  if ( nIdx < length()   &&  length() )
    truncate( nIdx );

  return *this;
}


EString &
EString::trimSpaces()
{
  trimLeft();
  return trimRight();
}

void 
EString::setHexa( UL nNum,
                  const char *sPrefix,
                  SL nNumDigit,
                  SL nLowerDigitNo )
{
  EASSERT( nNumDigit > 0 );        // must have at least one digit !
  EASSERT( nLowerDigitNo >= 0 );   

  if ( sPrefix )
    copy( sPrefix );

  SL nNumDigitCpt;
  SL nShift = (nLowerDigitNo + nNumDigit -1) *4;
  for ( nNumDigitCpt =0; nNumDigitCpt < nNumDigit; ++nNumDigitCpt, nShift -= 4 )
  {
    SL nDigit = (nNum >> nShift) & 15;      // digit to write

    if ( nDigit < 10 )
      append( (char)(nDigit + '0') );
    else
      append( (char)(nDigit + 'a' - 10) );
  }
}


void 
EString::format( const char *szFormat,
                 ... )
{
  if ( !szFormat )
    return;

  va_list argList;
  va_start( argList, szFormat );

  const SL nSizeStr =512;
  TCHAR szTmp[ nSizeStr +1 ];
  if ( ::_vsntprintf( szTmp, 
                      nSizeStr +1, 
                      szFormat, 
                      argList ) < 0 )             // Buffer big enough ?
  {
    SL nSize = 1024;
    SL nTry;
    for ( nTry =0; nTry < 28; ++nTry )
    {
      resize( nSize );
      if ( ::_vsntprintf( string(), 
                          nSize+1, 
                          szFormat, 
                          argList ) >= 0 )             // Buffer big enough ?
      {
        resize( _tcslen( string() ) );
        break;
      }
      else                                            // Buffer not big enough
        nSize *= 2;
    }
  }
  else
    copy( szTmp );

  va_end( argList );
}



// Storage operation
// ##########################################################################


EDataStream &
operator << ( EDataStream &s, const EString &str )
{
  s << str.length();
  s.writeBlock( str.datas(), str.length() );
  return s;
}


EDataStream &
operator >> ( EDataStream &s, EString &str )
{
  SL nLength;
  s >> nLength;
  str.resize( nLength );
  s.readBlock( &str.at(0), nLength );
  return s;
}




// ########################### EOF ESTRING.CPP ###############################
// ########################### EOF ESTRING.CPP ###############################
// ########################### EOF ESTRING.CPP ###############################


