#include "stdafx.h"
#include "EBitmap.h"

/* History:
- 13/01/98: Creation. Loading & Drawing of a transparent bitmap using mask & image.
*/


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Constructs the bitmap
CEBitmap::CEBitmap()
{
  init();
}

// Constructs the bitmap
// nIdBitmap: Id ressource bitmap ( fond = blanc )
// nIdBitmapMask: Id ressource mask bitmap (fond = blanc, mask = noir )
CEBitmap::CEBitmap( int nIdBitmap, 
                    int nIdBitmapMask )
{
  init();
  if ( nIdBitmap )
  {
    ASSERT( nIdBitmapMask );
    LoadBitmap( nIdBitmap, nIdBitmapMask );
  }
}

CEBitmap::~CEBitmap()
{
  if ( m_bmLoaded )
  {
    m_bm.DeleteObject();
    m_bmMask.DeleteObject();
  }
}

// Draw the bitmap on the DC.
// dc: DC the bitmap is drawn on.
// (x,y): position where the bitmap is drawn.
// xPos: aligment of the bitmap relatively to (x,y).
//         posLeft: x is the left border coordinate of the bitmap
//         posCenter: x is the center coordinate of the bitmap
//         posRight: x is the right border coordinate of the bitmap
// yPos: aligment of the bitmap relatively to (x,y).
//        posLeft: y is the top border coordinate of the bitmap
//        posCenter: y is the center coordinate of the bitmap
//        posRight: y is the top border coordinate of the bitmap
void CEBitmap::Draw(CDC& dc, int x, int y, int xPos, int yPos)
{
  if ( m_bmLoaded )
  {
    x = ComputeAbsPos( x, m_width, xPos );
    y = ComputeAbsPos( y, m_height, yPos );

    CDC dcBitmap;
    VERIFY( dcBitmap.CreateCompatibleDC(&dc) );

    CBitmap *oBitmap = dcBitmap.SelectObject( &m_bmMask );
    dc.BitBlt( x,y, m_width,m_height,
               &dcBitmap, 0,0,
               SRCAND );

    dcBitmap.SelectObject( &m_bm );
    dc.BitBlt( x,y, m_width,m_height,
               &dcBitmap, 0,0,
               SRCINVERT );

    dc.SelectObject( oBitmap );
  }
}

// Load the bitmap & the mask bitmap
// return TRUE on success.
BOOL CEBitmap::LoadBitmap( int nIdBitmap, 
                           int nIdBitmapMask )
{
  ASSERT( nIdBitmap != 0 );
  ASSERT( nIdBitmapMask != 0 );

  if ( !m_bm.LoadBitmap( nIdBitmap ) ) 
    return FALSE;
  if ( !m_bmMask.LoadMappedBitmap( nIdBitmapMask,
                                   CMB_MASKED) ) 
  {
    m_bm.DeleteObject();
    return FALSE;
  }
  
  m_bmLoaded = TRUE;
  BITMAP bmInfo;
  m_bm.GetBitmap( &bmInfo );
  m_width = bmInfo.bmWidth;
  m_height = bmInfo.bmHeight;
  ASSERT( m_width > 0 );
  ASSERT( m_height > 0 );
  return TRUE;
}


int CEBitmap::ComputeAbsPos(int nPos, int nSize, int nAlignement) const
{
  switch ( nAlignement )
  {
  case posRight:
      return nPos - nSize+1;
  case posCenter:
      return nPos -nSize/2;
  case posLeft:
  default:
      return nPos;
  }
}

void CEBitmap::init()
{
  m_bmLoaded = FALSE;
  m_width = 0;
  m_height = 0;
}
