// EwgScrollView.cpp : implementation file
//

/* History:
 * 09/02/1999: - Add InvalidateRect() & ValidateRect() to fix the window 95 bug
 *               when the rectangle height or width is greater than 32767 !
 *               Those two methods now works in Logical Coordinates.
 */


#include "stdafx.h"
#include "EwgScrollView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// EwgScrollView

IMPLEMENT_DYNCREATE(EwgScrollView, CView)

EwgScrollView::EwgScrollView()
{
}

EwgScrollView::~EwgScrollView()
{
}


BEGIN_MESSAGE_MAP(EwgScrollView, CView)
	//{{AFX_MSG_MAP(EwgScrollView)
	ON_WM_VSCROLL()
	ON_WM_HSCROLL()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// EwgScrollView drawing

void EwgScrollView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();
	// TODO: add draw code here
}

/////////////////////////////////////////////////////////////////////////////
// EwgScrollView diagnostics

#ifdef _DEBUG
void EwgScrollView::AssertValid() const
{
	CView::AssertValid();
}

void EwgScrollView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// EwgScrollView message handlers
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////



void 
EwgScrollView::OnVScroll( UINT nSBCode, 
                          UINT nPos, 
                          CScrollBar* pScrollBar ) 
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     // eat it

	// ignore scroll bar msgs from other controls
	if (pScrollBar != GetScrollBarCtrl(SB_VERT))
		return;

	OnScroll(MAKEWORD(-1, nSBCode), nPos);
}


void 
EwgScrollView::OnHScroll( UINT nSBCode, 
                          UINT nPos, 
                          CScrollBar* pScrollBar ) 
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     // eat it

	// ignore scroll bar msgs from other controls
	if (pScrollBar != GetScrollBarCtrl(SB_HORZ))
		return;

	OnScroll(MAKEWORD(nSBCode, -1), nPos);
}


BOOL 
EwgScrollView::OnScroll( UINT nScrollCode, 
                         UINT nPosBugged, 
                         BOOL bDoScroll ) 
{                                   // nPos is 16 bits and is therefore bugged !
	SCROLLINFO si;
	si.cbSize = sizeof(si);
	VERIFY(GetScrollInfo(SB_HORZ, &si, SIF_ALL));

  int nHPos = si.nPos;
  int nOldHPos = nHPos;

  switch ( LOBYTE(nScrollCode) )    // HScrollBar code
  {
  case SB_THUMBPOSITION:
    break;
  case SB_THUMBTRACK:
    nHPos = si.nTrackPos;
    break;
  case SB_LINEDOWN:
    nHPos += m_sizeLineScrollBar.cx;
    break;
  case SB_LINEUP:
    nHPos -= m_sizeLineScrollBar.cx;
    break;
  case SB_PAGEDOWN:
    nHPos += si.nPage;
    break;
  case SB_PAGEUP:
    nHPos -= si.nPage;
    break;
  case SB_BOTTOM:
    nHPos = si.nMax;
    break;
  case SB_TOP:
    nHPos = si.nMin;
    break;
  }
  nHPos = max( si.nMin, min( si.nMax, nHPos ) );



  VERIFY(GetScrollInfo(SB_VERT, &si, SIF_ALL));
  int nVPos = si.nPos;
  int nOldVPos = nVPos;

  switch ( HIBYTE(nScrollCode) )
  {
  case SB_THUMBPOSITION:
    break;
  case SB_THUMBTRACK:
    nVPos = si.nTrackPos;
    break;
  case SB_LINEDOWN:
    nVPos += m_sizeLineScrollBar.cy;
    break;
  case SB_LINEUP:
    nVPos -= m_sizeLineScrollBar.cy;
    break;
  case SB_PAGEDOWN:
    nVPos += si.nPage;
    break;
  case SB_PAGEUP:
    nVPos -= si.nPage;
    break;
  case SB_BOTTOM:
    nVPos = si.nMax;
    break;
  case SB_TOP:
    nVPos = si.nMin;
    break;
  }
  nVPos = max( si.nMin, min( si.nMax, nVPos ) );

  BOOL bScrolled;
  bScrolled = OnScrollBy( CSize( nHPos -nOldHPos, nVPos -nOldVPos ), 
                          bDoScroll );
	if ( bScrolled && bDoScroll )
		UpdateWindow();

	return bScrolled;
}



BOOL EwgScrollView::OnScrollBy(CSize sizeScroll, BOOL bDoScroll) 
{
  int nOldHPos, nHPos;
  int nOldVPos, nVPos;

	// don't scroll if there is no valid scroll range (ie. no scroll bar)
	CScrollBar* pBar;
	DWORD dwStyle = GetStyle();
	
  pBar = GetScrollBarCtrl(SB_HORZ);
	if ( (pBar != NULL  &&  !pBar->IsWindowEnabled() )  ||
		   (pBar == NULL  &&  !(dwStyle & WS_HSCROLL) ) )
	{
		// horizontal scroll bar not enabled
		sizeScroll.cx = 0;
  }

  pBar = GetScrollBarCtrl(SB_VERT);
	if ( ( pBar != NULL  &&  !pBar->IsWindowEnabled() )  ||
		   ( pBar == NULL  &&  !( dwStyle & WS_VSCROLL ) ) )
	{ // vertical scroll bar not enabled
		sizeScroll.cy = 0;
  }


	SCROLLINFO si;
	si.cbSize = sizeof(si);

  // adjust current nHPos position
	VERIFY(GetScrollInfo(SB_HORZ, &si, SIF_POS | SIF_RANGE ));
	nOldHPos = si.nPos;
	nHPos = max( si.nMin, min( si.nMax, nOldHPos+sizeScroll.cx ) );

  // adjust current nHPos position
  VERIFY(GetScrollInfo(SB_VERT, &si, SIF_POS | SIF_RANGE ));
	nOldVPos = si.nPos;
	nVPos = max( si.nMin, min( si.nMax, nOldVPos+sizeScroll.cy ) );

	// did anything change?
	if ( nHPos == nOldHPos  &&  nVPos == nOldVPos )
		return FALSE;

	if (bDoScroll)
	{
		// do scroll and update scroll positions
		ScrollWindow( -( nHPos -nOldHPos), 
                  -( nVPos -nOldVPos) );

    if ( nHPos != nOldHPos )
			SetScrollPos( SB_HORZ, nHPos );
		if ( nVPos != nOldVPos )
			SetScrollPos( SB_VERT, nVPos );
  }
  return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
// EwgScrollView methods
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////



void 
EwgScrollView::SetScrollSizes( const SIZE & sizeDoc, 
                               const SIZE & sizePage, 
                               const SIZE & sizeLine)
{
  SCROLLINFO info;
  info.cbSize = sizeof( info );
  info.fMask = SIF_ALL;
  info.nMin = 0;
  info.nMax = sizeDoc.cx;
  info.nPos = 0;
  info.nPage = sizePage.cx;
  VERIFY( SetScrollInfo( SB_HORZ, &info ) );

  info.fMask = SIF_ALL;
  info.nMin = 0;
  info.nMax = sizeDoc.cy;
  info.nPos = 0;
  info.nPage = sizePage.cy;
  VERIFY( SetScrollInfo( SB_VERT, &info ) );

  m_sizeLineScrollBar = sizeLine;
}


CPoint 
EwgScrollView::GetScrollPosition()
{   // Might use this later for smooth scrolling.
/*
  SCROLLINFO sih;
	sih.cbSize = sizeof(sih);
	SCROLLINFO siv;
	siv.cbSize = sizeof(siv);

  // adjust current nHPos position
	VERIFY(GetScrollInfo(SB_HORZ, &sih, SIF_POS | SIF_RANGE ));
	VERIFY(GetScrollInfo(SB_VERT, &siv, SIF_POS | SIF_RANGE ));

	CPoint pt( sih.nPos, 
             siv.nPos );
*/
	CPoint pt( GetScrollPos(SB_HORZ), 
             GetScrollPos(SB_VERT) );

	return pt;
}


void
EwgScrollView::ScrollToPosition( const CPoint &pt )
{
	SCROLLINFO si;
	si.cbSize = sizeof(si);

  // adjust current nHPos position
	VERIFY(GetScrollInfo(SB_HORZ, &si, SIF_POS | SIF_RANGE ));
	int nOldHPos = si.nPos;
	int nHPos = max( si.nMin, min( si.nMax, pt.x ) );

  // adjust current nHPos position
  VERIFY(GetScrollInfo(SB_VERT, &si, SIF_POS | SIF_RANGE ));
	int nOldVPos = si.nPos;
	int nVPos = max( si.nMin, min( si.nMax, pt.y ) );

	// did anything change?
	if ( nHPos == nOldHPos  &&  nVPos == nOldVPos )
		return;

	SetScrollPos( SB_HORZ, nHPos );
	SetScrollPos( SB_VERT, nVPos );
	ScrollWindow( nOldHPos -nHPos, 
                nOldVPos -nVPos );
}


void 
EwgScrollView::InvalidateRect( LPCRECT lpRect, 
                               BOOL bErase )
{
  if ( lpRect )
  {
    CPoint point = GetScrollPosition();
    RECT rect;
    rect.left = lpRect->left - point.x;
    rect.top = lpRect->top - point.y;
    rect.right = lpRect->right - point.x;
    rect.bottom = lpRect->bottom - point.y;

    if ( rect.bottom < 0  ||  rect.right < 0 )
      return;

    int nHeight = rect.bottom - rect.top;
    if ( nHeight > 32000 )
    {
      if ( rect.top < 0 )
      {
        nHeight += rect.top;
        rect.top = 0;
      }
      rect.bottom = nHeight > 32000 ? 32000 : nHeight;
    }

    int nWidth = rect.right - rect.left;
    if ( nWidth > 32000 )
    {
      if ( rect.left < 0 )
      {
        nWidth += rect.left;
        rect.left = 0;
      }
      rect.right = nWidth > 32000 ? 32000 : nWidth;
    }
    
    CView::InvalidateRect( &rect, bErase );
  }
  else
    CView::InvalidateRect( NULL, bErase );
}


void 
EwgScrollView::ValidateRect( LPCRECT lpRect )
{
  if ( lpRect )
  {
    CPoint point = GetScrollPosition();
    RECT rect;
    rect.left = lpRect->left - point.x;
    rect.top = lpRect->top - point.y;
    rect.right = lpRect->right - point.x;
    rect.bottom = lpRect->bottom - point.y;

    if ( rect.bottom < 0  ||  rect.right < 0 )
      return;

    int nHeight = rect.bottom - rect.top;
    if ( nHeight > 32000 )
    {
      if ( rect.top < 0 )
      {
        nHeight += rect.top;
        rect.top = 0;
      }
      rect.bottom = nHeight > 32000 ? 32000 : nHeight;
    }

    int nWidth = rect.right - rect.left;
    if ( nWidth > 32000 )
    {
      if ( rect.left < 0 )
      {
        nWidth += rect.left;
        rect.left = 0;
      }
      rect.right = nWidth > 32000 ? 32000 : nWidth;
    }
    
    CView::ValidateRect( &rect );
  }
  else
    CView::ValidateRect( NULL );
}
