#if !defined(AFX_EWGSCROLLVIEW_H__180BBAEA_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
#define AFX_EWGSCROLLVIEW_H__180BBAEA_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// EwgScrollView.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// EwgScrollView view

/*! This class provides a non scaled scroll view.
 *
 * Why this class ? \c CScrollView has a major limitation: the size of the document
 * is limited to 32768 with Windows 95 & there is also a small bug in the MFC with
 * NT that limit the thumb track movement to 0-32768 !
 *
 * The use of this class is similar to \c CScrollView but you must do coordinate change
 * by hand (the 32768 limitation come from device coordinate limitation !).
 *
 * Therefore in your \c OnLButtonDown(...) you will do:
 * \code
 * point += GetScrollPosition();
 * \endcode
 *
 * And in \c OnPaint() you will translate coordinate by \c -GetScrollPosition():
 * \code
 * CPoint pointMy( 10,20 );     // point in doc coord.
 * 
 * pointMy -= GetScrollPosition();    // point in client coord.
 * \endcode
 */
class EwgScrollView : public CView
{
protected:
	EwgScrollView();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(EwgScrollView);

// Attributes
public:
  /*! Set the size of the view.
   * @param sizeTotal Total size of the document.
   * @param sizePage Size of a page of the document (used when cliking outside 
   *                 the thumbtrack).
   * @param sizeLine Size of a line (used when clicking on the scroll arrow).
   */
	void SetScrollSizes( const SIZE &sizeTotal, 
                       const SIZE &sizePage, 
                       const SIZE &sizeLine );

  /*! Get the coordinates of the top-left corner of the window.
   * @return Coordinate of the top-left corner of the window.
   */
  CPoint GetScrollPosition();

  /*! Set the coordinates of the top-left corner of the window.
   * @param pt New coordinate of the top-left corner of the window.
   */
  void ScrollToPosition( const CPoint &pt );

// Operations
public:
  /*! Invalidates the client area within the given rectangle.
   * @param lpRect Rectangle to update. Coordinates are in logical unit (0 is not
   *               the top of the client window but the top of the document).
   *               If NULL then the entire client area is updated.
   * @param bErase If TRUE then the background of the client are is to be erased.
   */
  void InvalidateRect( LPCRECT lpRect, 
                       BOOL bErase = TRUE );

  /*! Validates the client area within the given rectangle.
   * @param lpRect Rectangle to validate. Coordinates are in logical unit (0 is not
   *               the top of the client window but the top of the document).
   *               If NULL then the entire client area is updated.
   */
  void ValidateRect( LPCRECT lpRect );


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(EwgScrollView)
	public:
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual BOOL OnScrollBy(CSize sizeScroll, BOOL bDoScroll = TRUE);
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~EwgScrollView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
protected:
	//{{AFX_MSG(EwgScrollView)
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP();

private:
  /*! Size of a line for the scroll view.
   */
  CSize m_sizeLineScrollBar;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_EWGSCROLLVIEW_H__180BBAEA_91B8_11D2_AF19_93F5DFEBAA61__INCLUDED_)
