// //////////////////////////////////////////////////////////////////////////
// Implementation file AssertTest.cpp for class AssertTest
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/10/10
// //////////////////////////////////////////////////////////////////////////
#include "Assert.h"
#include "AssertTest.h"
#include "CollectionAsserter.h"


CPPUNIT_TEST_SUITE_REGISTRATION( AssertTest );


AssertTest::AssertTest() : 
    CppUnit::TestCase()
{
}


AssertTest::~AssertTest()
{
}


void 
AssertTest::setUp()
{
}


void 
AssertTest::tearDown()
{
}


void 
AssertTest::testEqualsIntInt()
{
  GAIACRTN_ASSERT_EQUAL( 3, 3 );
}


void 
AssertTest::testEqualsDoubleInt()
{
  GAIACRTN_ASSERT_EQUAL( 3.0, 3 );
}


void 
AssertTest::testEqualsStdStringConstChar()
{
  GAIACRTN_ASSERT_EQUAL( std::string("abcdef"), "abcdef" );
}


void 
AssertTest::testSameEmptyCollections()
{
  std::vector<int> empty1;
  std::deque<int> empty2;
  GAIACRTN_ASSERT_SAME_COLLECTION( empty1, empty2 );
}


void 
AssertTest::testSameUnorderedCollections()
{
  static int expectedValues[5] = { 1,2,3,4,5 };
  std::vector<int> expected( expectedValues, expectedValues+5 );

  static int actualValues[5] = { 2,5,1,4,3 };
  std::vector<int> actual( actualValues, actualValues+5 );

  GAIACRTN_ASSERT_SAME_COLLECTION( expected, actual );
}


void 
AssertTest::testSameCollectionsMissingExpected()
{
  std::vector<int> some;
  some.push_back( 1 );
  std::deque<int> empty2;
  GAIACRTN_ASSERT_SAME_COLLECTION( some, empty2 );
}


void 
AssertTest::testSameCollectionsExtraneousActual()
{
  std::vector<int> empty;
  std::deque<int> extraneous;
  extraneous.push_back( 1 );
  GAIACRTN_ASSERT_SAME_COLLECTION( empty, extraneous );
}


void 
AssertTest::testSameUnorderedCollectionWithMissingAndExtraneousItems()
{
  static int expectedValues[] = { 2,3,4,7,8 };
  std::vector<int> expected( expectedValues, expectedValues+5 );

  static int actualValues[5] = { 2,4,3,8,10 };
  std::vector<int> actual( actualValues, actualValues+5 );

  GAIACRTN_ASSERT_SAME_COLLECTION( expected, actual );
}
