// //////////////////////////////////////////////////////////////////////////
// Header file AssertTest.h for class AssertTest
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/10/10
// //////////////////////////////////////////////////////////////////////////
#ifndef GAIACRTN_ASSERTTEST_H
#define GAIACRTN_ASSERTTEST_H

#include <cppunit/extensions/HelperMacros.h>


/*! \class AssertTest
 * \brief Unit test for Assert.
 */
class AssertTest : public CppUnit::TestCase
{
  CPPUNIT_TEST_SUITE( AssertTest );
  CPPUNIT_TEST( testEqualsIntInt );
  CPPUNIT_TEST( testEqualsDoubleInt );
  CPPUNIT_TEST( testEqualsStdStringConstChar );
  CPPUNIT_TEST( testSameEmptyCollections );
  CPPUNIT_TEST( testSameUnorderedCollections );
  CPPUNIT_TEST_FAIL( testSameCollectionsMissingExpected );
  CPPUNIT_TEST_FAIL( testSameCollectionsExtraneousActual );
  CPPUNIT_TEST_FAIL( testSameUnorderedCollectionWithMissingAndExtraneousItems );
  CPPUNIT_TEST_SUITE_END();

public:
  /*! Constructs a AssertTest object.
   */
  AssertTest();

  /// Destructor.
  virtual ~AssertTest();

  void setUp();
  void tearDown();

  void testEqualsIntInt();
  void testEqualsDoubleInt();
  void testEqualsStdStringConstChar();

  void testSameEmptyCollections();
  void testSameUnorderedCollections();
  void testSameCollectionsMissingExpected();
  void testSameCollectionsExtraneousActual();
  void testSameUnorderedCollectionWithMissingAndExtraneousItems();

private:
  /// Prevents the use of the copy constructor.
  AssertTest( const AssertTest &other );

  /// Prevents the use of the copy operator.
  void operator =( const AssertTest &other );

private:
};



// Inlines methods for AssertTest:
// -------------------------------



#endif  // GAIACRTN_ASSERTTEST_H
