// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtPara.h for class TtrFmtPara
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/14
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTPARA_H
#define TTRFMTPARA_H

#include "TtrFmtElt.h"
#include "TtrFmtIndentInfo.h"
#include "TtrTextLineFormat.h"

class TtrFmtPara;

/*! Auto-pointer on formatted text elements.
 */
typedef EAutoPtrT<TtrFmtPara> TtrFmtParaPtr;


/*! \class TtrFmtPara
 * \brief This class represents a paragraph of a formatted text.
 */
class TtrFmtPara : public TtrFmtElt
{
public:
  /*! Constructs a TtrFmtPara object.
   */
  TtrFmtPara();

  /*! Constructs a TtrFmtPara object.
   */
  TtrFmtPara( const EString &strContent,
              SL nIndentLevel =0 );

  /*! Destructor.
   */
  virtual ~TtrFmtPara();

  /*! Gets the type of the element.
   * @return Type of the element.
   */
  virtual EltType type() const;

  /*! Visits the element.
   * @param visitor Visitor that is visited by the element.
   */
  virtual void visit( TtrFmtEltVisitor &visitor );

  /*! Sets the indent information.
   * @param infoIndent Indent information.
   */
  void setIndent( const TtrFmtIndentInfo &infoIndent );

  /*! Returns the indent information.
   * @return Indent information.
   */
  const TtrFmtIndentInfo &indent() const;

  /*! Sets the paragraph content.
   * @param strContent Paragraph content.
   */
  void setContent( const EString &strContent );

  /*! Returns the content of the paragraph.
   * @returns Content of the paragraph.
   */
  const EString &content() const;

  const TtrTextLineFormat &format() const;
  
  void setFormat( const TtrTextLineFormat &format );


  bool operator ==( const TtrFmtPara &cmpTo ) const;
  bool operator !=( const TtrFmtPara &cmpTo ) const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtPara( const TtrFmtPara &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtPara &operator =( const TtrFmtPara &copy );

private:
  TtrFmtIndentInfo _infoIndent;
  EString _strContent;
  TtrTextLineFormat _format;
};



// Inlines methods for TtrFmtPara:
// -------------------------------


inline const EString &
TtrFmtPara::content() const
{
  return _strContent;
}


inline void 
TtrFmtPara::setIndent( const TtrFmtIndentInfo &infoIndent )
{
  _infoIndent = infoIndent;
}


inline void 
TtrFmtPara::setContent( const EString &strContent )
{
  _strContent = strContent;
}


inline const TtrFmtIndentInfo &
TtrFmtPara::indent() const
{
  return _infoIndent;
}


inline const TtrTextLineFormat &
TtrFmtPara::format() const
{
  return _format;
}


inline void 
TtrFmtPara::setFormat( const TtrTextLineFormat &format )
{
  _format = format;
}



#endif  // TTRFMTPARA_H
