// //////////////////////////////////////////////////////////////////////////
// Header file TtrFmtText.h for class TtrFmtText
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/14
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRFMTTEXT_H
#define TTRFMTTEXT_H

#include "TtrFmtElt.h"


/*! \class TtrFmtText
 * \brief This class represents a formatted text.
 */
class TtrFmtText
{
public:
  /*! Constructs a TtrFmtText object.
   */
  TtrFmtText();

  /*! Destructor.
   */
  virtual ~TtrFmtText();

  /*! Returns the number of element in the text.
   * @return Number of element in the text.
   */
  SL eltCount() const;

  /*! Returns the element at the specified index.
   * @param nIdx Index of the element.
   * @return Reference on the element at position \a nIdx.
   */
  const TtrFmtElt &eltAt( SL nIdx ) const;

  /*! Returns the element at the specified index.
   * @param nIdx Index of the element.
   * @return Reference on the element at position \a nIdx.
   */
  TtrFmtElt &eltAt( SL nIdx );

  /*! Adds an element.
   * @param pElt Pointer on the element to add.
   */
  void addElt( TtrFmtEltPtr &ptrElt );

  /*! Takes the element at the specified index.
   * @param nIdx Indent of the element to take.
   * @return Auto-pointer on the element. \c NULL if \a nIdx is out of bound.
   */
  TtrFmtEltPtr takeEltAt( SL nIdx );

  /*! Removes all the element.
   */
  void removeAll();

  /*! Visits all the elements of the text with the specified visitor.
   * @param visitor Visitor being visited.
   */
  void visit( TtrFmtEltVisitor &visitor );

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrFmtText( const TtrFmtText &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrFmtText &operator =( const TtrFmtText &copy );

private:
  typedef EPtrDynArrayT<TtrFmtElt *> ArrayElt;

  ArrayElt _vpElt;
};



// Inlines methods for TtrFmtText:
// -------------------------------


inline SL 
TtrFmtText::eltCount() const
{
  return _vpElt.count();
}


inline const TtrFmtElt &
TtrFmtText::eltAt( SL nIdx ) const
{
  return *_vpElt[ nIdx ];
}


inline TtrFmtElt &
TtrFmtText::eltAt( SL nIdx )
{
  return *_vpElt[ nIdx ];
}


#endif  // TTRFMTTEXT_H
