// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrFmtTextBuilder.cpp for class TtrFmtTextBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/16
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrFmtTextBuilder.h"
#include "TtrFmtText.h"
#include "TtrFmtBlank.h"
#include "TtrFmtPara.h"
#include "TtrWriterSetting.h"


// //////////////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////////////
// class TtrFmtTextBuilder
// //////////////////////////////////////////////////////////////////////////
// //////////////////////////////////////////////////////////////////////////


TtrFmtTextBuilder::TtrFmtTextBuilder( TtrFmtText &text,
                                      const TtrWriterSetting &setting ) :
    _pText( &text ),
    _bRemoveEndingHyphen( setting.getWriterRemoveEndingHyphen() ),
    _bJumpOneLine( setting.getWriterJumpOneLine() )
{
  makeCurrentBlockInvalid();
}


TtrFmtTextBuilder::~TtrFmtTextBuilder()
{
}


void 
TtrFmtTextBuilder::changeLevel( SL nBlockLevel,
                                const EString &strLevelPrefix )
{
  endBlock();
  
  _infoBlockDepth._nDepth = nBlockLevel;
  _infoBlockDepth._strPrefix = strLevelPrefix;
}


void 
TtrFmtTextBuilder::newBlock( const EString &strContent,
                             SL nIndentCount,
                             const EString &strIndent,
                             const TtrTextLineFormat &format )
{
  EASSERT( nIndentCount >= 0 );

  endBlock();

  _infoBlock._nIndentCount = nIndentCount;
  _infoBlock._strIndent = strIndent;
  _infoBlock._strContent = strContent;
  _infoBlock._strContent.trimSpaces();
  _infoBlock._format = format;
}


void 
TtrFmtTextBuilder::appendToBlock( const EString &strTextToAppend )
{
  EASSERT( isCurrentBlockValid() ); // newBlock() was not called!

  EString strText( strTextToAppend );
  strText.trimSpaces();

  if ( _bRemoveEndingHyphen  &&
       _infoBlock._strContent.lastChar() == '-'   )
  {
    _infoBlock._strContent.resize( _infoBlock._strContent.length() -1 );
  }
  else if ( _infoBlock._strContent.length() > 0  &&
            strText.length() > 0 )
  {
    _infoBlock._strContent.append( ' ' );
  }

  _infoBlock._strContent.append( strText );
}


void 
TtrFmtTextBuilder::endBlock()
{
  if ( !isCurrentBlockValid() )
    return;

  if ( _infoBlock._strContent.length() > 0 )
    addParagraphToText();
  else
    addBlankToText();
}


void
TtrFmtTextBuilder::addParagraphToText()
{
  TtrFmtIndentInfo indentInfo( _infoBlock._nIndentCount, 
                               _infoBlockDepth._strPrefix );
  
  // Until indent string is added to indentInfo.
  EString strContent = _infoBlock._strIndent;
  strContent += _infoBlock._strContent;

  TtrFmtParaPtr ptrPara( new TtrFmtPara( strContent ) );
  ptrPara->setIndent( indentInfo );
  ptrPara->setFormat( _infoBlock._format );

  if ( _bJumpOneLine  &&  !isTextEmpty() )
    addBlankToText();

  _pText->addElt( TtrFmtEltPtr( ptrPara.detach() ) );
}


bool
TtrFmtTextBuilder::isTextEmpty() const
{
  return _pText->eltCount() == 0;
}



void
TtrFmtTextBuilder::addBlankToText()
{
  if ( _pText->eltCount() > 0 )
  {
    TtrFmtElt &lastElt = _pText->eltAt( _pText->eltCount() -1 );
    TtrFmtBlank *pLastBlank = dynamic_cast<TtrFmtBlank *>( &lastElt );
    if ( pLastBlank != NULL )
    {
      pLastBlank->addLine( TtrTextLineFormat() );
      return;
    }
  }

  TtrFmtIndentInfo indentInfo( 0, _infoBlockDepth._strPrefix );
  TtrFmtEltPtr ptrBlank( new TtrFmtBlank( indentInfo, 
                                          TtrTextLineFormat() ) );
  _pText->addElt( ptrBlank );
}


bool 
TtrFmtTextBuilder::isCurrentBlockValid() const
{
  return _infoBlock._nIndentCount >= 0;
}


void 
TtrFmtTextBuilder::makeCurrentBlockInvalid()
{
  _infoBlock._nIndentCount = -1;
}
