// //////////////////////////////////////////////////////////////////////////
// Header file TtrTagAtt.h for class TtrTagAtt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/09
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRTAGATT_H
#define TTRTAGATT_H


class TtrTagAtt;

/*! Type for reference on object of type TtrTagAtt.
 */
typedef ERef<TtrTagAtt> TtrTagAttRef;


/*! \class TtrTagAtt
 * \brief This class represents an attribut of a tag.
 */
class TtrTagAtt : public ERefObj
{
public:
  /*! Constructs a TtrTagAtt object.
   * @param strAttName Name of the attribut.
   * @param strAttValue Value of the attribut. If the string is empty, the it
   *                    it set to the name of the attribut.
   */
  TtrTagAtt( const EString &strAttName,
             const EString &strAttValue );

  /*! Destructor.
   */
  virtual ~TtrTagAtt();

  /*! Get the name of the attribut.
   * @return Name of the attribut.
   */
  const EString &attName() const;

  /*! Get the value of the attribut.
   * @return Attribut value.
   */
  const EString &attValue() const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrTagAtt( const TtrTagAtt &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrTagAtt &operator =( const TtrTagAtt &copy );

private:

  /*! Id of the attribut name.
   */
  const ESymbolId _sidAttName;

  /*! Attribute value.
   */
  const EString _strAttValue;
};



// Inlines methods for TtrTagAtt:
// ------------------------------


inline const EString &
TtrTagAtt::attValue() const
{
  return _strAttValue;
}



#endif  // TTRTAGATT_H
