// Generated by Together

#ifndef TTRTEXTLINE_H
#define TTRTEXTLINE_H

#include "TtrTextLineFormat.h"


class TtrTextLine : public EString
{
public:
  static TtrTextLine &getNullLine();

  /** Constructs an empty line.*/
  TtrTextLine();

  /** Constructs a line with a given text.*/
  TtrTextLine(const EString &strText);      
  
  /** Destructor.*/
  virtual ~TtrTextLine() {};
  
  /** Set the text of the line.
   * @param strText Text of the line.
   */
  void setText( const EString &strText );

  /** Get  the text of the line.*/
  const EString & text() const;   

  /** Returns the line without the indent string.
   * @return Line without the indent string.
   */
  EString content() const;
  
  /** Test if the line is a separator (---- or **** or ====)
   * @param strSeparator String of char which are separator.
   * @return TRUE if so.
   */
  bool isSeparatorLine( const EString &strSeparator ) const;  
    
  /** Get the number of trailing space on the line.*/
  SL countTrailingSpace() const;       
  
  /** Get the number of heading space on the line.*/
  SL countHeadingSpace() const;

  /** Get the position of the first trailing space. */
  SL firstTrailingSpacePos() const;
  
  /** Set the indentation level of the line.
   * @param nIndentLevel New indentation level.*/
  void setIndentLevel( SL nIndentLevel = 0 );      
  
  /** Decrease the indentation level.
   * @param nDec Decreasing count.*/
  SL decIndentLevel( SL nDec =1 );          
  
  /** Increase the indentation level.
   * @param nInc Increasing count.*/
  SL incIndentLevel( SL nInc = 1);           
  
  /** Test if the indentation level is > 0.*/
  bool isAppended() const;                                  
  
  /** Test if the indentation level of the line is null.*/
  bool isNormal() const;                                     
  
  /** Indentation level of the line : _nIndentLevel*/
  SL indentLevel() const;
  
  /** Get the number of space character at the beginning of the line.*/
  SL indentCount() const;                                      

  /** Test if the line is a blank line.
   * 
   * @return \c true if the line contains only space characters, 
   *         otherwise returns \c false.
   */
  bool isBlank() const;

  /** Get the first non-white char of the line.
   *
   * @return The first non-white char of the line. 0 if none.
   */
  TCHAR firstNonWhiteChar() const;

  /** Is the line a composite text marker.
   * @return \c true if the line is a composite text markern otherwise returns \c false.
   *         A TtrTextLine always returns \c false.
   */
  bool isCompositeMarker() const;

  /*! Defines the length of the indent string.
   * @param nIndentLength Length of the indent string.
   */
  void setIndentLength( SL nIndentLength =0 );

  /*! Returns the indent string length.
   * @return Indent string length.
   */
  SL indentLength() const;

  TtrTextLineFormat format() const;

  void setFormat( const TtrTextLineFormat &format );

private:
 /** Indentation level of the line:
  * 0 :  not indented
  * >0 : The line is indented N times.
  * <0: the line is appended to the previous line.*/
  SL _nIndentLevel;

  /*! Length of the indent string.
   * If =0 then the line does not have an indent string.
   */
  SL _nIndentLength;
  
  TtrTextLineFormat _format;

  static TtrTextLine _lineNull;
};




// Inlines TtrTextLine:
// --------------------

inline TtrTextLine &
TtrTextLine::getNullLine()
{
  return _lineNull;
}


inline
TtrTextLine::TtrTextLine(const EString &strText ) :
    EString( strText),
    _nIndentLevel( 0 ),
    _nIndentLength( 0 )
{
}


inline SL 
TtrTextLine::indentLevel() const 
{
  return _nIndentLevel;
}


inline bool 
TtrTextLine::isNormal() const 
{
  return indentLevel() == 0;
}


inline bool 
TtrTextLine::isAppended() const 
{
  return indentLevel() < 0;
}


inline SL 
TtrTextLine::incIndentLevel( SL nInc )
{
  _nIndentLevel += nInc;
  return _nIndentLevel;
}


inline SL 
TtrTextLine::decIndentLevel( SL nDec )
{
  _nIndentLevel -= nDec;
  return _nIndentLevel;
}


inline void 
TtrTextLine::setIndentLevel( SL nIndentLevel )
{
  _nIndentLevel = nIndentLevel;
}


inline SL 
TtrTextLine::indentCount() const 
{
  return countHeadingSpace();
}


inline const EString & 
TtrTextLine::text() const 
{
  return *this;
}


inline void 
TtrTextLine::setText( const EString &strText )
{
  copy( strText );
}


inline 
TtrTextLine::TtrTextLine() :
    _nIndentLevel( 0 )
{
}


inline bool 
TtrTextLine::isBlank() const
{
  return indentCount() == length();
}


inline SL 
TtrTextLine::countTrailingSpace() const 
{
  return length() - firstTrailingSpacePos();
}


inline TCHAR 
TtrTextLine::firstNonWhiteChar() const
{
  SL nPos = countHeadingSpace();
  return nPos < length() ? at( nPos ) : 0;
}


inline void 
TtrTextLine::setIndentLength( SL nIndentLength )
{
  _nIndentLength = nIndentLength;
}


inline SL 
TtrTextLine::indentLength() const
{
  return _nIndentLength;
}


inline TtrTextLineFormat 
TtrTextLine::format() const
{
  return _format;
}


inline void 
TtrTextLine::setFormat( const TtrTextLineFormat &format )
{
  _format = format;
}



#endif //TTRTEXTLINE_H
