// //////////////////////////////////////////////////////////////////////////
// Header file TtrTextReformatterBuilder.h for class TtrTextReformatterBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/04/14
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRTEXTREFORMATTERBUILDER_H
#define TTRTEXTREFORMATTERBUILDER_H

class TtrTextLineFormat;


/*! \class TtrTextReformatterBuilder
 * \brief This class is a base class for builder of reformatted text.
 */
class TtrTextReformatterBuilder
{
public:
  /*! Destructor.
   */
  virtual ~TtrTextReformatterBuilder();

  /** Changes the current block level.
   * @param nBlockLevel Depth of the next block.
   * @param strLevelPrefix Prefix string for the next block.
   */
  virtual void changeLevel( SL nBlockLevel,
                            const EString &strLevelPrefix ) =0;

  /**  Starts a new block using the specified content and indentation.
   * @param strContent Content of the new block. The content is trimmed
   *                   before being used for the block.
   * @param nIndentCount Number of indent to apply at the beginning
   *                     of the block. If 0 then \a strIndentString
   *                     it used to make the indentation. Must be >=0.
   * @param strIndentString String used if \a nIndentCount is 0 to
   *                        indent the new block.
   * @param format Format of the new block.
   */
  virtual void newBlock( const EString &strContent,
                         SL nIndentCount,
                         const EString &strIndent,
                         const TtrTextLineFormat &format ) =0;

  /** Appends the specified text to the current block.
   *
   * The specified text is trimmed then appended to the current block
   * using the hyphen rule.
   *
   * newBlock() must be called before any call to appendToBlock().
   *
   * @param strTextToAppend Text to append to the current block.
   */
  virtual void appendToBlock( const EString &strTextToAppend ) =0;

  /** Ends the current block.
   */
  virtual void endBlock() =0;
};



// Inlines methods for TtrTextReformatterBuilder:
// ----------------------------------------------



#endif  // TTRTEXTREFORMATTERBUILDER_H
