// //////////////////////////////////////////////////////////////////////////
// Implementation file TtrUtFmtTextBuilder.cpp for class TtrUtFmtTextBuilder
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2001/04/29
// //////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "TtrUtFmtTextBuilder.h"
#include "TtrFmtBlank.h"
#include "TtrFmtPara.h"
#include "TtrFmtText.h"
#include "TtrFmtTextBuilder.h"
#include "TtrGlobalSetting.h"


CPPUNIT_TEST_SUITE_REGISTRATION( TtrUtFmtTextBuilder );


TtrUtFmtTextBuilder::TtrUtFmtTextBuilder()
{
}


TtrUtFmtTextBuilder::~TtrUtFmtTextBuilder()
{
}


CppUnit::TestSuite *
TtrUtFmtTextBuilder::suite()
{
  CppUnit::TestSuiteBuilder<TtrUtFmtTextBuilder> suite;

  suite.addTestCaller( "testChangeLevel0", testChangeLevel0 );
  suite.addTestCaller( "testChangeLevel2", testChangeLevel2 );
  suite.addTestCaller( "testIsTextEmpty", testIsTextEmpty );
  suite.addTestCaller( "testAddBlankToText", testAddBlankToText );
  suite.addTestCaller( "testAddIndentedParagraphToText", testAddIndentedParagraphToText );
  suite.addTestCaller( "testAddPrefixedParagraphToText", testAddPrefixedParagraphToText );
  suite.addTestCaller( "testNewBlock", testNewBlock );
  suite.addTestCaller( "testAppendToBlock", testAppendToBlock );
  suite.addTestCaller( "testAppendToBlockUsingHyphenRule", testAppendToBlockUsingHyphenRule );
  suite.addTestCaller( "testEndBlockForPara", testEndBlockForPara );
  suite.addTestCaller( "testEndBlockForBlank", testEndBlockForBlank );
  suite.addTestCaller( "testJumpOneLine", testJumpOneLine );

  return suite.takeSuite();
}


void 
TtrUtFmtTextBuilder::setUp()
{
  _pText = new TtrFmtText();
  
  _pSetting = new TtrGlobalSetting();
  _pSetting->setWriterRemoveEndingHyphen( true );
  _pSetting->setWriterJumpOneLine( false );

  _pSettingJumpLine = new TtrGlobalSetting();
  _pSettingJumpLine->setWriterRemoveEndingHyphen( false );
  _pSettingJumpLine->setWriterJumpOneLine( true );

  _pBuilder = new TtrFmtTextBuilder( *_pText, *_pSetting );
  
  _pBuilderJumpLine = new TtrFmtTextBuilder( *_pText, *_pSettingJumpLine );
}


void 
TtrUtFmtTextBuilder::tearDown()
{
  delete _pBuilder;
  delete _pText;
  delete _pSettingJumpLine;
  delete _pSetting;
}


void
TtrUtFmtTextBuilder::testChangeLevel0()
{
  _pBuilder->changeLevel( 0, "" );
  GAIACRTN_ASSERT_EQUAL( 0, _pBuilder->_infoBlockDepth._nDepth );
  GAIACRTN_ASSERT_EQUAL( "",
                         _pBuilder->_infoBlockDepth._strPrefix );
}


void
TtrUtFmtTextBuilder::testChangeLevel2()
{
  _pBuilder->changeLevel( 2, ">>" );
  GAIACRTN_ASSERT_EQUAL( 2, _pBuilder->_infoBlockDepth._nDepth );
  GAIACRTN_ASSERT_EQUAL( ">>", _pBuilder->_infoBlockDepth._strPrefix );
}


void 
TtrUtFmtTextBuilder::testIsTextEmpty()
{
  GAIACRTN_ASSERT( _pBuilder->isTextEmpty() );
  _pText->addElt( TtrFmtEltPtr( new TtrFmtPara( "sometext" ) ) );
  GAIACRTN_ASSERT( !_pBuilder->isTextEmpty() );
}


void 
TtrUtFmtTextBuilder::testAddBlankToText()
{
  _pBuilder->addBlankToText();
  TtrFmtBlank &blank = dynamic_cast<TtrFmtBlank &>( _pText->eltAt(0) );
  GAIACRTN_ASSERT_EQUAL( 1, blank.lineCount() );

  _pBuilder->addBlankToText();
  TtrFmtBlank &blank2 = dynamic_cast<TtrFmtBlank &>( _pText->eltAt(0) );
  GAIACRTN_ASSERT_EQUAL( 2, blank2.lineCount() );
  GAIACRTN_ASSERT_EQUAL( 1, _pText->eltCount() );
}


void 
TtrUtFmtTextBuilder::testAddIndentedParagraphToText()
{
  _pBuilder->_infoBlock._strIndent = "\t";
  _pBuilder->_infoBlock._strContent = "A text";
  _pBuilder->_infoBlock._nIndentCount = 2;
  _pBuilder->_infoBlock._format = TtrTextLineFormat().centerOn();
  _pBuilder->addParagraphToText();

  TtrFmtPara &para = dynamic_cast<TtrFmtPara &>( _pText->eltAt( 0 ) );
  GAIACRTN_ASSERT_EQUAL( "\tA text", para.content() );
  GAIACRTN_ASSERT_EQUAL( 2, para.indent().indentLevel() );
  GAIACRTN_ASSERT_EQUAL( "", para.indent().indentString() );
  GAIACRTN_ASSERT_EQUAL( TtrTextLineFormat().centerOn(), para.format() );
}


void 
TtrUtFmtTextBuilder::testAddPrefixedParagraphToText()
{
  _pBuilder->changeLevel( 1, ">" );
  _pBuilder->_infoBlock._strIndent = "";
  _pBuilder->_infoBlock._strContent = "A text";
  _pBuilder->_infoBlock._nIndentCount = 1;
  _pBuilder->addParagraphToText();

  TtrFmtPara &para = dynamic_cast<TtrFmtPara &>( _pText->eltAt( 0 ) );
  GAIACRTN_ASSERT_EQUAL( "A text", para.content() );
  GAIACRTN_ASSERT_EQUAL( 1, para.indent().indentLevel() );
  GAIACRTN_ASSERT_EQUAL( ">", para.indent().indentString() );
}


void 
TtrUtFmtTextBuilder::testNewBlock()
{
  addIndentedAndCenteredNewBlock( _pBuilder );
  GAIACRTN_ASSERT_EQUAL( 1, _pBuilder->_infoBlock._nIndentCount );
  GAIACRTN_ASSERT_EQUAL( "A new indented line", 
                         _pBuilder->_infoBlock._strContent );
  GAIACRTN_ASSERT_EQUAL( "  ", _pBuilder->_infoBlock._strIndent );
  GAIACRTN_ASSERT_EQUAL( TtrTextLineFormat().centerOn(), 
                         _pBuilder->_infoBlock._format );
}


void 
TtrUtFmtTextBuilder::addIndentedAndCenteredNewBlock( TtrFmtTextBuilder *pBuilder )
{
  pBuilder->newBlock( "  A new indented line ",
                      1,
                      "  ",
                      TtrTextLineFormat().centerOn() );
}


void 
TtrUtFmtTextBuilder::testAppendToBlock()
{
  addIndentedAndCenteredNewBlock( _pBuilder );
  _pBuilder->appendToBlock( "   next part of  " );
  GAIACRTN_ASSERT_EQUAL( "A new indented line next part of",
                         _pBuilder->_infoBlock._strContent );
}


void 
TtrUtFmtTextBuilder::testAppendToBlockUsingHyphenRule()
{
  _pBuilder->newBlock( "  A new indented li- ",
                       1,
                       "  ",
                       TtrTextLineFormat().centerOn() );
  _pBuilder->appendToBlock( "  ne.  " );
  GAIACRTN_ASSERT_EQUAL( "A new indented line.", 
                         _pBuilder->_infoBlock._strContent );
}


void 
TtrUtFmtTextBuilder::testEndBlockForPara()
{
  _pBuilder->changeLevel( 0, "" );
  _pText->removeAll();

  _pBuilder->_infoBlock._strIndent = "  ";
  _pBuilder->_infoBlock._strContent = "A text";
  _pBuilder->_infoBlock._nIndentCount = 1;
  _pBuilder->endBlock();
  GAIACRTN_ASSERT( _pText->eltCount() > 0 );
  TtrFmtPara &para = dynamic_cast<TtrFmtPara &>( _pText->eltAt( 0 ) );
  GAIACRTN_ASSERT_EQUAL( "  A text", para.content() );
  GAIACRTN_ASSERT_EQUAL( 1, para.indent().indentLevel() );
}


void 
TtrUtFmtTextBuilder::testEndBlockForBlank()
{
  _pBuilder->changeLevel( 0, "" );
  _pText->removeAll();

  _pBuilder->_infoBlock._strIndent = "";
  _pBuilder->_infoBlock._strContent = "";
  _pBuilder->_infoBlock._nIndentCount = 1;
  _pBuilder->endBlock();
  GAIACRTN_ASSERT( _pText->eltCount() > 0 );
  TtrFmtBlank &blank = dynamic_cast<TtrFmtBlank &>( _pText->eltAt( 0 ) );
  GAIACRTN_ASSERT_EQUAL( 1, blank.lineCount() );
}


void 
TtrUtFmtTextBuilder::testJumpOneLine()
{
  _pBuilderJumpLine->newBlock( "First line", 1, "", TtrTextLineFormat() );
  _pBuilderJumpLine->newBlock( "Second line", 1, "", TtrTextLineFormat() );
  _pBuilderJumpLine->endBlock();

  GAIACRTN_ASSERT_EQUAL( 3, _pText->eltCount() );
  
  TtrFmtPara &para1 = dynamic_cast<TtrFmtPara &>( _pText->eltAt( 0 ) );
  GAIACRTN_ASSERT_EQUAL( "First line", para1.content() );

  TtrFmtBlank &blank = dynamic_cast<TtrFmtBlank &>( _pText->eltAt( 1 ) );
  GAIACRTN_ASSERT_EQUAL( 1, blank.lineCount() );

  TtrFmtPara &para2 = dynamic_cast<TtrFmtPara &>( _pText->eltAt( 2 ) );
  GAIACRTN_ASSERT_EQUAL( "Second line", para2.content() );
}
