// TtrUtWriter.cpp: implementation of the TtrUtWriter class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TtrUtWriter.h"
#include "TtrTextRaw.h"
#include "EwgProgressInfo.h"
#include "TtrTextWriterTxt.h"
#include "TtrTextWriterHtml.h"
#include "TtrTagDef.h"
#include "TtrTagManager.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif




void AFX_CDECL MyTrace(LPCTSTR lpszFormat, ...)
{
#ifdef _DEBUG // all AfxTrace output is controlled by afxTraceEnabled
	if (!afxTraceEnabled)
		return;
#endif

	va_list args;
	va_start(args, lpszFormat);

	int nBuf;
	TCHAR szBuffer[16384];

	nBuf = _vsntprintf(szBuffer, sizeof(szBuffer)/sizeof(TCHAR), lpszFormat, args);

	if ((afxTraceFlags & traceMultiApp) && (AfxGetApp() != NULL))
		afxDump << AfxGetApp()->m_pszExeName << ": ";
	afxDump << szBuffer;

	va_end(args);
}


const EString g_strDirTest( "test\\" );


CU_TEST_SUITE_REGISTRATION( TtrUtWriter );


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TtrUtWriter::TtrUtWriter()
{
  _pText = new TtrTextRaw();
  _pProgressInfo = new EwgProgressInfo();

  static EString strText(
    "Maximum Polarity - Balance of Power\n"     //0
    "A BGC Fan-fiction by Arthur Hansen (Triaxm'l) arthurh@utah-inter.net\n"
    "\n"
    "       Nene, stretched and yawned, and turned back to her computer.  It was\n" //3
    "late, after   midnight. Naoko was already gone, and it was probably time for\n"
    "her to head out herself. It   had actually been a slow day, and she had a\n"
    "normal day off tomorrow. No Knight Saber   business either, yay! It should\n" //6
    "be heaven. A sigh escaped her lips. Even work was better. It   was nice that\n"
    "the ADP had finally got to computers on the desks of the office workers.\n"
    "With some of the trips that she had installed, she could get some flags to\n"  //9
    "pop up, and she   wouldn't have to leave her desk to get some of the more\n"
    "basic information for the Knight   Sabers.\n"
    "       She had even put in some alarms about her, in case somebody dug in\n"  //12
    "too deep, and   figured her out. It was something that she put in after Dr.\n"
    "Mirriam's attack on the ADP,   when a girl named Lisa figured out that she\n"
    "was a Knight Saber. That was too close, Nene   thought. Ah, well, grab my\n"  //15
    "jacket, and I'm out of here. Just as her hand reached her coat,   her\n"
    "computer beeped, in a seemingly random fashion. \"Uh, oh.\" Nene said as she\n"
    "stared at   her computer across the room.\n"  //18
    "       That was probably the biggest understatement ever uttered by Nene.\n"
    "Sliding   around to look at the screen, she saw her future at the ADP\n"
    "disintegrate. \"Oh, no!\" she   moaned as she read a memo being sent to the\n"  //21
    "front desk concerning her arrest as a mole   for the Knight Sabers. Nene\n"
    "flinched when she saw the part of the memo that warns that   she is probably\n"
    "armed and dangerous.  Shoot to capture. A quick glance revealed the\n"  //24
    "officer in charge of her capture. Matio Dansk, a known Genom mole. \"I'd\n"
    "better get out of   here. Dansk is probably already on his way over!\"\n"
    "       With a flurry of keystrokes, she sent out a handful of specialized\n" //27
    "commands that   would cripple the ADP for about an hour. And more. If she\n"
    "couldn't get out in that time,   she wasn't getting out at all. The last\n"
    "command sent a message to the Knight Sabers about   her cover being blown.\n"  //30
    "And with that, she fled out the door to the stairs, now invisible to   the\n"
    "cameras and sensors.\n"
    "       -\n"  //33
    "       Officer Dansk knew that he had his mole now. She couldn't get away,\n"
    "and his true   employers were more than willing to pay him enough for him to\n"
    "retire. \"Lock and load   gentlemen. She might not look dangerous, but it's a\n" //36
    "high probability that she's actually one   of the Knight Sabers moles. If\n"
    "she comes along quietly, then all the better.\" Dansk stated   loudly enough\n"
    "for the two officers to hear. One of them just looked at him, and shrug-\n"  //36
    "ged his shoulders.\n"  //40
  );
  setText( strText );
}


void 
TtrUtWriter::reformattedText()
{
  makeOriginal( 0, 2 );
  makePara( 2, 1 );
  makePara( 3, 9 );
  makePara( 12, 7 );
  makePara( 19, 8 );
  makePara( 27, 6 );
  makePara( 33, 1 );
  makePara( 34, 7 );
  _pText->line( 33 ).setFormat( TtrTextLineFormat().centerOn() );
}


void 
TtrUtWriter::originalText()
{
  makeOriginal( 0, 41 );
}


TtrUtWriter::~TtrUtWriter()
{
  delete _pProgressInfo;
  delete _pText;
}


void 
TtrUtWriter::makePara( int nStartLine, int nCount )
{
  _pText->line( nStartLine++ ).setIndentLevel( 1 );
  for ( --nCount; nCount-- > 0; ++nStartLine )
    _pText->line( nStartLine ).setIndentLevel( -1 );
}


void 
TtrUtWriter::makeOriginal( int nStartLine, int nCount )
{
  for ( ; nCount-- > 0; ++nStartLine )
    _pText->line( nStartLine ).setIndentLevel( 0 );
}


void
TtrUtWriter::setText( const EString &strText )
{
  _pText->clear();
  SL nPos = 0;
  do
  {
    SL nNextPos = strText.pos( nPos, '\n' );
    SL nBreak;
    nBreak = ( nNextPos == -1 ) ? strText.length() : 
                                  nBreak = nNextPos;

    EString strLine( strText.mid( nPos, nBreak-nPos ) );    // don't include '\n'
    _pText->insertLine( new TtrTextLine( strLine ) );

    if ( nNextPos == -1 )
      break;

    nPos = nNextPos+1;    // skip the '\n'
  }
  while ( nPos < strText.length() );
}


void 
TtrUtWriter::saveAndCheckTxt( const EString &strTestText )
{
  EBuffer buffer;
  buffer.open( EIO_ReadWrite );
  TtrTextWriterTxt writer;
  writer.writeText( _mode, _mode, *_pText, buffer, *_pProgressInfo );

  buffer.at( 0 );

  {
    // Save result to file
    EFile fileSave( g_strDirTest + "gen-" + strTestText );
    fileSave.open( EIO_WriteOnly );
    while ( !buffer.atEnd() )
      fileSave.putByte( buffer.getByte() );
    fileSave.close();
    buffer.at( 0 );
  }


  EFile file( g_strDirTest + strTestText );
  file.open( EIO_ReadOnly );
  ETextStream streamTest( &file );
  ETextStream streamGenerated( &buffer );
  int nLineNo =0;
  ETRY
  {
    do
    {
      EString strLineTest;
      EString strLineGenerated;

      if ( streamTest.device()->atEnd() )
      {
        ETRY
        {
          streamGenerated.readLine( strLineGenerated );
          MyTrace( "Expected EOF line %d for %s (generated):\n", 
                   nLineNo, 
                   (LPCTSTR)strTestText.cstr() );
          MyTrace( "GEN: %s\n", (LPCTSTR)strLineGenerated.cstr() );
          GAIACRTN_FAIL( "EOF not synchronized !" );
        }
        ECATCH( EXCIODevice, e )
        {
          GAIACRTN_ASSERT_EQUAL( EXCIODevice::endOfFile, e.cause() );
          return;
        }
      }

      streamTest.readLine( strLineTest );
      streamGenerated.readLine( strLineGenerated );

      if ( strLineTest != strLineGenerated )
      {
        MyTrace( "Different line %d for %s:\n", nLineNo, (LPCTSTR)strTestText.cstr() );
        MyTrace( "TST: %s\n", (LPCTSTR)strLineTest.cstr() );
        MyTrace( "GEN: %s\n", (LPCTSTR)strLineGenerated.cstr() );
        GAIACRTN_ASSERT_EQUAL( strLineTest, strLineGenerated );
      }

      ++nLineNo;
    }
    while ( true );
  }
  ECATCH( EXCIODevice, e )
  {
    if ( e.cause() != EXCIODevice::endOfFile )
      ETHROW_AGAIN;

    MyTrace( "Unexpected EOF line %d for %s (generated):\n", 
             nLineNo, 
             (LPCTSTR)strTestText.cstr() );
    GAIACRTN_FAIL( "Unexpected end of file" );
  }
}


void 
TtrUtWriter::writerTxtOriginal()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentAsOriginal );
  _mode.setWriterReindentSize( 13 );
  _mode.setWriterJumpOneLine( false );
  _mode.setWriterRemoveEndingHyphen( false );
  originalText();
  saveAndCheckTxt( "TtrUtWriter-1-unchanged.txt" );
}


void 
TtrUtWriter::writerTxtIndentTab()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentWithTab );
  _mode.setWriterReindentSize( 1 );
  _mode.setWriterJumpOneLine( false );
  _mode.setWriterRemoveEndingHyphen( true );
  reformattedText();
  saveAndCheckTxt( "TtrUtWriter-1-indent-tab.txt" );
}


void 
TtrUtWriter::writerTxtIndent3Tab()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentWithTab );
  _mode.setWriterReindentSize( 3 );
  _mode.setWriterJumpOneLine( false );
  _mode.setWriterRemoveEndingHyphen( true );
  reformattedText();
  saveAndCheckTxt( "TtrUtWriter-1-indent-3tab.txt" );
}


void 
TtrUtWriter::writerTxtIndentSpace()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentWithSpace );
  _mode.setWriterReindentSize( 1 );
  _mode.setWriterJumpOneLine( false );
  _mode.setWriterRemoveEndingHyphen( true );
  reformattedText();
  saveAndCheckTxt( "TtrUtWriter-1-indent-space.txt" );
}


void 
TtrUtWriter::writerTxtIndent3Space()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentWithSpace );
  _mode.setWriterReindentSize( 3 );
  _mode.setWriterJumpOneLine( false );
  _mode.setWriterRemoveEndingHyphen( true );
  reformattedText();
  saveAndCheckTxt( "TtrUtWriter-1-indent-3space.txt" );
}


void 
TtrUtWriter::writerTxtIndentTabJumpOneLine()
{
  _mode.setWriterType( wstpV0 );
  _mode.setWriterLineMode( wslmIndentWithTab );
  _mode.setWriterReindentSize( 1 );
  _mode.setWriterJumpOneLine( true );
  _mode.setWriterRemoveEndingHyphen( true );
  reformattedText();
  saveAndCheckTxt( "TtrUtWriter-1-indent-tab-jump-one-line.txt" );
}
