// //////////////////////////////////////////////////////////////////////////
// Header file TtrWriterTxt.h for class TtrWriterTxt
// (c)Copyright 2000, Baptiste Lepilleur.
// Created: 2000/09/17
// //////////////////////////////////////////////////////////////////////////
#ifndef TTRWRITERTXT_H
#define TTRWRITERTXT_H


#include "TtrFmtEltVisitor.h"
class TtrWriterSetting;


/*! \class TtrWriterTxt
 * \brief This class represents a writer that write a formatted text in a text file.
 */
class TtrWriterTxt : public TtrFmtEltVisitor
{
public:
  /*! Constructs a TtrWriterTxt object.
   * @param rstreamOut Output text stream.
   * @param setting Writer setting.
   */
  TtrWriterTxt( ETextStream &rstreamOut,
                const TtrWriterSetting &setting );

  /*! Destructor.
   */
  virtual ~TtrWriterTxt();

  /*! Visits a blank block.
   * @param blank Blank block.
   */
  virtual void visitBlank( TtrFmtBlank &blank );

  /*! Visits a paragraph.
   * @param para Paragraph.
   */
  virtual void visitPara( TtrFmtPara &para );

  /*! Should be called when the writer is started.
   */
  void start();

  /*! Should be called when the writer is done.
   */
  void end();

  /*! Returns the text stream used by the writer.
   * @return Text stream used by the writer.
   */
  ETextStream &stream() const;

private:
  /*! Prevents the use of the copy constructor.
   */
  TtrWriterTxt( const TtrWriterTxt &copy );

  /*! Prevents the use of the copy operator.
   */
  TtrWriterTxt &operator =( const TtrWriterTxt &copy );

  /*! Setups the writer using the writer setting.
   * @param setting Writer setting.
   */
  void setup( const TtrWriterSetting &setting );
private:
  /*! Should we keep the original indentation on indented line ?
   */
  bool _bKeepOriginalIndent;

  /*! String for an indentation level "step".
   */
  EString _strIndentStep;

  /*! Reference on the output text stream.
   */
  ETextStream &_rstreamOut;
};



// Inlines methods for TtrWriterTxt:
// ---------------------------------


inline ETextStream &
TtrWriterTxt::stream() const
{
  return _rstreamOut;
}



#endif  // TTRWRITERTXT_H
